from dataclasses import dataclass
from typing import Any, Dict, List, Optional, Union
from datetime import datetime
import dataclasses

@dataclass
class ProviderDeploymentsConfigsUpdateOutputDeployment:
    object: str
    id: str
    is_default: bool
    provider_id: str
    created_at: datetime
    updated_at: datetime
    name: Optional[str] = None
    description: Optional[str] = None
    metadata: Optional[Dict[str, Any]] = None
@dataclass
class ProviderDeploymentsConfigsUpdateOutputFromVaultDeployment:
    object: str
    id: str
    is_default: bool
    provider_id: str
    created_at: datetime
    updated_at: datetime
    name: Optional[str] = None
    description: Optional[str] = None
    metadata: Optional[Dict[str, Any]] = None
@dataclass
class ProviderDeploymentsConfigsUpdateOutputFromVault:
    object: str
    id: str
    name: str
    provider_id: str
    created_at: datetime
    updated_at: datetime
    description: Optional[str] = None
    metadata: Optional[Dict[str, Any]] = None
    deployment: Optional[ProviderDeploymentsConfigsUpdateOutputFromVaultDeployment] = None
@dataclass
class ProviderDeploymentsConfigsUpdateOutput:
    object: str
    id: str
    is_default: bool
    provider_id: str
    specification_id: str
    created_at: datetime
    updated_at: datetime
    name: Optional[str] = None
    description: Optional[str] = None
    metadata: Optional[Dict[str, Any]] = None
    deployment: Optional[ProviderDeploymentsConfigsUpdateOutputDeployment] = None
    from_vault: Optional[ProviderDeploymentsConfigsUpdateOutputFromVault] = None


class mapProviderDeploymentsConfigsUpdateOutputDeployment:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ProviderDeploymentsConfigsUpdateOutputDeployment:
        return ProviderDeploymentsConfigsUpdateOutputDeployment(
        object=data.get('object'),
        id=data.get('id'),
        is_default=data.get('is_default'),
        name=data.get('name'),
        description=data.get('description'),
        metadata=data.get('metadata'),
        provider_id=data.get('provider_id'),
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None,
        updated_at=datetime.fromisoformat(data.get('updated_at').replace('Z', '+00:00')) if data.get('updated_at') else None
        )

    @staticmethod
    def to_dict(value: Union[ProviderDeploymentsConfigsUpdateOutputDeployment, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapProviderDeploymentsConfigsUpdateOutputFromVaultDeployment:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ProviderDeploymentsConfigsUpdateOutputFromVaultDeployment:
        return ProviderDeploymentsConfigsUpdateOutputFromVaultDeployment(
        object=data.get('object'),
        id=data.get('id'),
        is_default=data.get('is_default'),
        name=data.get('name'),
        description=data.get('description'),
        metadata=data.get('metadata'),
        provider_id=data.get('provider_id'),
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None,
        updated_at=datetime.fromisoformat(data.get('updated_at').replace('Z', '+00:00')) if data.get('updated_at') else None
        )

    @staticmethod
    def to_dict(value: Union[ProviderDeploymentsConfigsUpdateOutputFromVaultDeployment, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapProviderDeploymentsConfigsUpdateOutputFromVault:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ProviderDeploymentsConfigsUpdateOutputFromVault:
        return ProviderDeploymentsConfigsUpdateOutputFromVault(
        object=data.get('object'),
        id=data.get('id'),
        name=data.get('name'),
        description=data.get('description'),
        metadata=data.get('metadata'),
        provider_id=data.get('provider_id'),
        deployment=mapProviderDeploymentsConfigsUpdateOutputFromVaultDeployment.from_dict(data.get('deployment')) if data.get('deployment') else None,
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None,
        updated_at=datetime.fromisoformat(data.get('updated_at').replace('Z', '+00:00')) if data.get('updated_at') else None
        )

    @staticmethod
    def to_dict(value: Union[ProviderDeploymentsConfigsUpdateOutputFromVault, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapProviderDeploymentsConfigsUpdateOutput:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ProviderDeploymentsConfigsUpdateOutput:
        return ProviderDeploymentsConfigsUpdateOutput(
        object=data.get('object'),
        id=data.get('id'),
        is_default=data.get('is_default'),
        name=data.get('name'),
        description=data.get('description'),
        metadata=data.get('metadata'),
        provider_id=data.get('provider_id'),
        specification_id=data.get('specification_id'),
        deployment=mapProviderDeploymentsConfigsUpdateOutputDeployment.from_dict(data.get('deployment')) if data.get('deployment') else None,
        from_vault=mapProviderDeploymentsConfigsUpdateOutputFromVault.from_dict(data.get('from_vault')) if data.get('from_vault') else None,
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None,
        updated_at=datetime.fromisoformat(data.get('updated_at').replace('Z', '+00:00')) if data.get('updated_at') else None
        )

    @staticmethod
    def to_dict(value: Union[ProviderDeploymentsConfigsUpdateOutput, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        # assume dataclass for generated models
        return dataclasses.asdict(value)

@dataclass
class ProviderDeploymentsConfigsUpdateBody:
    name: Optional[str] = None
    description: Optional[str] = None
    metadata: Optional[Dict[str, Any]] = None


class mapProviderDeploymentsConfigsUpdateBody:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ProviderDeploymentsConfigsUpdateBody:
        return ProviderDeploymentsConfigsUpdateBody(
        name=data.get('name'),
        description=data.get('description'),
        metadata=data.get('metadata')
        )

    @staticmethod
    def to_dict(value: Union[ProviderDeploymentsConfigsUpdateBody, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        # assume dataclass for generated models
        return dataclasses.asdict(value)




from typing import Any, Dict, List, Optional, Union
from metorial._endpoint import (
  BaseMetorialEndpoint,
  MetorialEndpointManager,
  MetorialRequest,
)
from ..resources import (
  mapDashboardOrganizationsProjectsListOutput,
  DashboardOrganizationsProjectsListOutput,
  mapDashboardOrganizationsProjectsListQuery,
  DashboardOrganizationsProjectsListQuery,
  mapDashboardOrganizationsProjectsGetOutput,
  DashboardOrganizationsProjectsGetOutput,
  mapDashboardOrganizationsProjectsCreateOutput,
  DashboardOrganizationsProjectsCreateOutput,
  mapDashboardOrganizationsProjectsCreateBody,
  DashboardOrganizationsProjectsCreateBody,
  mapDashboardOrganizationsProjectsDeleteOutput,
  DashboardOrganizationsProjectsDeleteOutput,
  mapDashboardOrganizationsProjectsUpdateOutput,
  DashboardOrganizationsProjectsUpdateOutput,
  mapDashboardOrganizationsProjectsUpdateBody,
  DashboardOrganizationsProjectsUpdateBody,
)


class MetorialDashboardOrganizationsProjectsEndpoint(BaseMetorialEndpoint):
  """Read and write project information"""

  def __init__(self, config: MetorialEndpointManager):
    super().__init__(config)

  def list(
    self,
    organization_id: str,
    *,
    limit: Optional[float] = None,
    after: Optional[str] = None,
    before: Optional[str] = None,
    cursor: Optional[str] = None,
    order: Optional[str] = None,
    team_id: Optional[Union[str, List[str]]] = None
  ) -> DashboardOrganizationsProjectsListOutput:
    """
    List organization projects
    List all organization projects

    :param organization_id: str
    :param limit: Optional[float] (optional)
    :param after: Optional[str] (optional)
    :param before: Optional[str] (optional)
    :param cursor: Optional[str] (optional)
    :param order: Optional[str] (optional)
    :param team_id: Optional[Union[str, List[str]]] (optional)
    :return: DashboardOrganizationsProjectsListOutput
    """
    # Build query parameters from keyword arguments
    query_dict = {}
    if limit is not None:
      query_dict["limit"] = limit
    if after is not None:
      query_dict["after"] = after
    if before is not None:
      query_dict["before"] = before
    if cursor is not None:
      query_dict["cursor"] = cursor
    if order is not None:
      query_dict["order"] = order
    if team_id is not None:
      query_dict["team_id"] = team_id

    request = MetorialRequest(
      path=["dashboard", "organizations", organization_id, "projects"], query=query_dict
    )
    return self._get(request).transform(
      mapDashboardOrganizationsProjectsListOutput.from_dict
    )

  def get(
    self, organization_id: str, project_id: str
  ) -> DashboardOrganizationsProjectsGetOutput:
    """
    Get organization project
    Get the information of a specific organization project

    :param organization_id: str
    :param project_id: str
    :return: DashboardOrganizationsProjectsGetOutput
    """
    request = MetorialRequest(
      path=["dashboard", "organizations", organization_id, "projects", project_id]
    )
    return self._get(request).transform(
      mapDashboardOrganizationsProjectsGetOutput.from_dict
    )

  def create(
    self, organization_id: str, *, name: str
  ) -> DashboardOrganizationsProjectsCreateOutput:
    """
    Create organization project
    Create a new organization project

    :param organization_id: str
    :param name: str
    :return: DashboardOrganizationsProjectsCreateOutput
    """
    # Build body parameters from keyword arguments
    body_dict = {}
    body_dict["name"] = name

    request = MetorialRequest(
      path=["dashboard", "organizations", organization_id, "projects"], body=body_dict
    )
    return self._post(request).transform(
      mapDashboardOrganizationsProjectsCreateOutput.from_dict
    )

  def delete(
    self, organization_id: str, project_id: str
  ) -> DashboardOrganizationsProjectsDeleteOutput:
    """
    Delete organization project
    Remove an organization project

    :param organization_id: str
    :param project_id: str
    :return: DashboardOrganizationsProjectsDeleteOutput
    """
    request = MetorialRequest(
      path=["dashboard", "organizations", organization_id, "projects", project_id]
    )
    return self._delete(request).transform(
      mapDashboardOrganizationsProjectsDeleteOutput.from_dict
    )

  def update(
    self, organization_id: str, project_id: str, *, name: Optional[str] = None
  ) -> DashboardOrganizationsProjectsUpdateOutput:
    """
    Update organization project
    Update the role of an organization project

    :param organization_id: str
    :param project_id: str
    :param name: Optional[str] (optional)
    :return: DashboardOrganizationsProjectsUpdateOutput
    """
    # Build body parameters from keyword arguments
    body_dict = {}
    if name is not None:
      body_dict["name"] = name

    request = MetorialRequest(
      path=["dashboard", "organizations", organization_id, "projects", project_id],
      body=body_dict,
    )
    return self._post(request).transform(
      mapDashboardOrganizationsProjectsUpdateOutput.from_dict
    )



"""
OpenAI provider adapter.
"""

from collections.abc import AsyncGenerator, Callable
from typing import TYPE_CHECKING, Any, cast

from metorial._executor import get_executor

from .base import ChatMessage, ChatResponse, ProviderAdapter
from .message_utils import convert_messages_to_openai, extract_openai_tool_calls

if TYPE_CHECKING:
  from metorial_openai import build_openai_tools, call_openai_tools
else:
  try:
    from metorial_openai import build_openai_tools, call_openai_tools
  except ImportError:
    call_openai_tools: Callable[[Any, list[Any]], list[dict[str, Any]]] | None = None
    build_openai_tools: Callable[[Any], list[dict[str, Any]]] | None = None


class OpenAIAdapter(ProviderAdapter):
  """Adapter for OpenAI-compatible providers"""

  async def create_chat_completion(
    self,
    messages: list[ChatMessage],
    tools: list[dict[str, Any]],
    model: str = "gpt-4o",
    **kwargs: Any,
  ) -> ChatResponse:
    # Convert standardized messages to OpenAI format
    openai_messages = convert_messages_to_openai(messages)

    # Handle both sync and async clients
    # Check if this is an async client by looking at the client type
    if hasattr(self.client, "__class__") and "Async" in self.client.__class__.__name__:
      # Async client
      response = await self.client.chat.completions.create(
        model=model, messages=openai_messages, tools=tools, **kwargs
      )
    else:
      # Sync client - run in shared thread pool
      executor = get_executor()
      future = executor.submit(
        self.client.chat.completions.create,
        model=model,
        messages=openai_messages,
        tools=tools,
        **kwargs,
      )
      response = future.result()

    # Convert OpenAI response to standardized format using shared utility
    tool_calls = extract_openai_tool_calls(response)

    return ChatResponse(
      content=response.choices[0].message.content if response.choices else None,
      tool_calls=tool_calls if tool_calls else None,
      usage=response.usage.dict() if response.usage else None,
    )

  async def create_chat_completion_stream(
    self,
    messages: list[ChatMessage],
    tools: list[dict[str, Any]],
    model: str = "gpt-4o",
    **kwargs: Any,
  ) -> AsyncGenerator[dict[str, Any], None]:
    # Convert standardized messages to OpenAI format
    openai_messages = convert_messages_to_openai(messages)

    # Handle both sync and async clients
    if hasattr(self.client, "__class__") and "Async" in self.client.__class__.__name__:
      # Async client
      stream = await self.client.chat.completions.create(
        model=model,
        messages=openai_messages,
        tools=tools,
        stream=True,
        **kwargs,
      )
    else:
      # Sync client - run in shared thread pool
      executor = get_executor()
      future = executor.submit(
        self.client.chat.completions.create,
        model=model,
        messages=openai_messages,
        tools=tools,
        stream=True,
        **kwargs,
      )
      stream = future.result()

    async for chunk in stream:
      if chunk.choices:
        choice = chunk.choices[0]
        delta = choice.delta

        if delta.content:
          yield {"type": "content", "content": delta.content}
        elif delta.tool_calls:
          for tool_call in delta.tool_calls:
            yield {"type": "tool_call", "tool_call": tool_call}

  async def call_tools(self, tool_calls: list[dict[str, Any]]) -> list[ChatMessage]:
    """Execute tool calls using OpenAI format"""
    if call_openai_tools is None:
      raise ImportError("metorial-openai package is required for OpenAI adapter")

    tool_messages = await call_openai_tools(self.tool_manager, tool_calls)
    return self._convert_tool_responses(tool_messages)

  def get_tools_for_provider(self) -> list[dict[str, Any]]:
    """Get tools in OpenAI format"""
    if build_openai_tools is None:
      raise ImportError("metorial-openai package is required for OpenAI adapter")

    return cast(list[dict[str, Any]], build_openai_tools(self.tool_manager))



from typing import Any, Dict, List, Optional, Union
from metorial._endpoint import (
  BaseMetorialEndpoint,
  MetorialEndpointManager,
  MetorialRequest,
)
from ..resources import (
  mapServersListingsCollectionsListOutput,
  ServersListingsCollectionsListOutput,
  mapServersListingsCollectionsListQuery,
  ServersListingsCollectionsListQuery,
  mapServersListingsCollectionsGetOutput,
  ServersListingsCollectionsGetOutput,
)


class MetorialServersListingsCollectionsEndpoint(BaseMetorialEndpoint):
  """Read and write server listing collection information"""

  def __init__(self, config: MetorialEndpointManager):
    super().__init__(config)

  def list(
    self,
    *,
    limit: Optional[float] = None,
    after: Optional[str] = None,
    before: Optional[str] = None,
    cursor: Optional[str] = None,
    order: Optional[str] = None
  ) -> ServersListingsCollectionsListOutput:
    """
    List server listing collections
    List all server listing collections

    :param limit: Optional[float] (optional)
    :param after: Optional[str] (optional)
    :param before: Optional[str] (optional)
    :param cursor: Optional[str] (optional)
    :param order: Optional[str] (optional)
    :return: ServersListingsCollectionsListOutput
    """
    # Build query parameters from keyword arguments
    query_dict = {}
    if limit is not None:
      query_dict["limit"] = limit
    if after is not None:
      query_dict["after"] = after
    if before is not None:
      query_dict["before"] = before
    if cursor is not None:
      query_dict["cursor"] = cursor
    if order is not None:
      query_dict["order"] = order

    request = MetorialRequest(path=["server-listing-collections"], query=query_dict)
    return self._get(request).transform(
      mapServersListingsCollectionsListOutput.from_dict
    )

  def get(
    self, server_listing_collection_id: str
  ) -> ServersListingsCollectionsGetOutput:
    """
    Get server listing collection
    Get the information of a specific server listing collection

    :param server_listing_collection_id: str
    :return: ServersListingsCollectionsGetOutput
    """
    request = MetorialRequest(
      path=["server-listing-collections", server_listing_collection_id]
    )
    return self._get(request).transform(
      mapServersListingsCollectionsGetOutput.from_dict
    )



from dataclasses import dataclass
from typing import Any, Dict, List, Optional, Union
from datetime import datetime
from metorial.utils import parse_iso_datetime
import dataclasses


@dataclass
class ServersDeploymentsGetOutputOauthConnectionProvider:
  id: str
  name: str
  url: str
  image_url: str


@dataclass
class ServersDeploymentsGetOutputOauthConnection:
  object: str
  id: str
  status: str
  name: str
  metadata: Dict[str, Any]
  provider: ServersDeploymentsGetOutputOauthConnectionProvider
  config: Dict[str, Any]
  client_id: str
  instance_id: str
  created_at: datetime
  updated_at: datetime
  description: Optional[str] = None
  template_id: Optional[str] = None


@dataclass
class ServersDeploymentsGetOutputCallbackSchedule:
  object: str
  interval_seconds: float
  next_run_at: datetime


@dataclass
class ServersDeploymentsGetOutputCallback:
  object: str
  id: str
  type: str
  schedule: ServersDeploymentsGetOutputCallbackSchedule
  created_at: datetime
  updated_at: datetime
  url: Optional[str] = None
  name: Optional[str] = None
  description: Optional[str] = None


@dataclass
class ServersDeploymentsGetOutputServer:
  object: str
  id: str
  name: str
  type: str
  created_at: datetime
  updated_at: datetime
  description: Optional[str] = None


@dataclass
class ServersDeploymentsGetOutputConfig:
  object: str
  id: str
  status: str
  secret_id: str
  created_at: datetime


@dataclass
class ServersDeploymentsGetOutputServerImplementationServerVariant:
  object: str
  id: str
  identifier: str
  server_id: str
  source: Dict[str, Any]
  created_at: datetime


@dataclass
class ServersDeploymentsGetOutputServerImplementationServer:
  object: str
  id: str
  name: str
  type: str
  created_at: datetime
  updated_at: datetime
  description: Optional[str] = None


@dataclass
class ServersDeploymentsGetOutputServerImplementation:
  object: str
  id: str
  status: str
  name: str
  metadata: Dict[str, Any]
  server_variant: ServersDeploymentsGetOutputServerImplementationServerVariant
  server: ServersDeploymentsGetOutputServerImplementationServer
  created_at: datetime
  updated_at: datetime
  description: Optional[str] = None
  get_launch_params: Optional[str] = None


@dataclass
class ServersDeploymentsGetOutputAccess:
  ip_allowlist: Dict[str, Any]


@dataclass
class ServersDeploymentsGetOutput:
  object: str
  id: str
  status: str
  name: str
  result: Dict[str, Any]
  metadata: Dict[str, Any]
  secret_id: str
  server: ServersDeploymentsGetOutputServer
  config: ServersDeploymentsGetOutputConfig
  server_implementation: ServersDeploymentsGetOutputServerImplementation
  created_at: datetime
  updated_at: datetime
  description: Optional[str] = None
  oauth_connection: Optional[ServersDeploymentsGetOutputOauthConnection] = None
  callback: Optional[ServersDeploymentsGetOutputCallback] = None
  access: Optional[ServersDeploymentsGetOutputAccess] = None


class mapServersDeploymentsGetOutputOauthConnectionProvider:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> ServersDeploymentsGetOutputOauthConnectionProvider:
    return ServersDeploymentsGetOutputOauthConnectionProvider(
      id=data.get("id"),
      name=data.get("name"),
      url=data.get("url"),
      image_url=data.get("image_url"),
    )

  @staticmethod
  def to_dict(
    value: Union[
      ServersDeploymentsGetOutputOauthConnectionProvider, Dict[str, Any], None
    ]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapServersDeploymentsGetOutputOauthConnection:
  @staticmethod
  def from_dict(data: Dict[str, Any]) -> ServersDeploymentsGetOutputOauthConnection:
    return ServersDeploymentsGetOutputOauthConnection(
      object=data.get("object"),
      id=data.get("id"),
      status=data.get("status"),
      name=data.get("name"),
      description=data.get("description"),
      metadata=data.get("metadata"),
      provider=mapServersDeploymentsGetOutputOauthConnectionProvider.from_dict(
        data.get("provider")
      )
      if data.get("provider")
      else None,
      config=data.get("config"),
      client_id=data.get("client_id"),
      instance_id=data.get("instance_id"),
      template_id=data.get("template_id"),
      created_at=parse_iso_datetime(data.get("created_at"))
      if data.get("created_at")
      else None,
      updated_at=parse_iso_datetime(data.get("updated_at"))
      if data.get("updated_at")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[ServersDeploymentsGetOutputOauthConnection, Dict[str, Any], None]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapServersDeploymentsGetOutputCallbackSchedule:
  @staticmethod
  def from_dict(data: Dict[str, Any]) -> ServersDeploymentsGetOutputCallbackSchedule:
    return ServersDeploymentsGetOutputCallbackSchedule(
      object=data.get("object"),
      interval_seconds=data.get("interval_seconds"),
      next_run_at=parse_iso_datetime(data.get("next_run_at"))
      if data.get("next_run_at")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[ServersDeploymentsGetOutputCallbackSchedule, Dict[str, Any], None]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapServersDeploymentsGetOutputCallback:
  @staticmethod
  def from_dict(data: Dict[str, Any]) -> ServersDeploymentsGetOutputCallback:
    return ServersDeploymentsGetOutputCallback(
      object=data.get("object"),
      id=data.get("id"),
      url=data.get("url"),
      name=data.get("name"),
      description=data.get("description"),
      type=data.get("type"),
      schedule=mapServersDeploymentsGetOutputCallbackSchedule.from_dict(
        data.get("schedule")
      )
      if data.get("schedule")
      else None,
      created_at=parse_iso_datetime(data.get("created_at"))
      if data.get("created_at")
      else None,
      updated_at=parse_iso_datetime(data.get("updated_at"))
      if data.get("updated_at")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[ServersDeploymentsGetOutputCallback, Dict[str, Any], None]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapServersDeploymentsGetOutputServer:
  @staticmethod
  def from_dict(data: Dict[str, Any]) -> ServersDeploymentsGetOutputServer:
    return ServersDeploymentsGetOutputServer(
      object=data.get("object"),
      id=data.get("id"),
      name=data.get("name"),
      description=data.get("description"),
      type=data.get("type"),
      created_at=parse_iso_datetime(data.get("created_at"))
      if data.get("created_at")
      else None,
      updated_at=parse_iso_datetime(data.get("updated_at"))
      if data.get("updated_at")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[ServersDeploymentsGetOutputServer, Dict[str, Any], None]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapServersDeploymentsGetOutputConfig:
  @staticmethod
  def from_dict(data: Dict[str, Any]) -> ServersDeploymentsGetOutputConfig:
    return ServersDeploymentsGetOutputConfig(
      object=data.get("object"),
      id=data.get("id"),
      status=data.get("status"),
      secret_id=data.get("secret_id"),
      created_at=parse_iso_datetime(data.get("created_at"))
      if data.get("created_at")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[ServersDeploymentsGetOutputConfig, Dict[str, Any], None]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapServersDeploymentsGetOutputServerImplementationServerVariant:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> ServersDeploymentsGetOutputServerImplementationServerVariant:
    return ServersDeploymentsGetOutputServerImplementationServerVariant(
      object=data.get("object"),
      id=data.get("id"),
      identifier=data.get("identifier"),
      server_id=data.get("server_id"),
      source=data.get("source"),
      created_at=parse_iso_datetime(data.get("created_at"))
      if data.get("created_at")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[
      ServersDeploymentsGetOutputServerImplementationServerVariant, Dict[str, Any], None
    ]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapServersDeploymentsGetOutputServerImplementationServer:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> ServersDeploymentsGetOutputServerImplementationServer:
    return ServersDeploymentsGetOutputServerImplementationServer(
      object=data.get("object"),
      id=data.get("id"),
      name=data.get("name"),
      description=data.get("description"),
      type=data.get("type"),
      created_at=parse_iso_datetime(data.get("created_at"))
      if data.get("created_at")
      else None,
      updated_at=parse_iso_datetime(data.get("updated_at"))
      if data.get("updated_at")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[
      ServersDeploymentsGetOutputServerImplementationServer, Dict[str, Any], None
    ]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapServersDeploymentsGetOutputServerImplementation:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> ServersDeploymentsGetOutputServerImplementation:
    return ServersDeploymentsGetOutputServerImplementation(
      object=data.get("object"),
      id=data.get("id"),
      status=data.get("status"),
      name=data.get("name"),
      description=data.get("description"),
      metadata=data.get("metadata"),
      get_launch_params=data.get("get_launch_params"),
      server_variant=mapServersDeploymentsGetOutputServerImplementationServerVariant.from_dict(
        data.get("server_variant")
      )
      if data.get("server_variant")
      else None,
      server=mapServersDeploymentsGetOutputServerImplementationServer.from_dict(
        data.get("server")
      )
      if data.get("server")
      else None,
      created_at=parse_iso_datetime(data.get("created_at"))
      if data.get("created_at")
      else None,
      updated_at=parse_iso_datetime(data.get("updated_at"))
      if data.get("updated_at")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[ServersDeploymentsGetOutputServerImplementation, Dict[str, Any], None]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapServersDeploymentsGetOutputAccess:
  @staticmethod
  def from_dict(data: Dict[str, Any]) -> ServersDeploymentsGetOutputAccess:
    return ServersDeploymentsGetOutputAccess(ip_allowlist=data.get("ip_allowlist"))

  @staticmethod
  def to_dict(
    value: Union[ServersDeploymentsGetOutputAccess, Dict[str, Any], None]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapServersDeploymentsGetOutput:
  @staticmethod
  def from_dict(data: Dict[str, Any]) -> ServersDeploymentsGetOutput:
    return ServersDeploymentsGetOutput(
      object=data.get("object"),
      id=data.get("id"),
      status=data.get("status"),
      name=data.get("name"),
      description=data.get("description"),
      oauth_connection=mapServersDeploymentsGetOutputOauthConnection.from_dict(
        data.get("oauth_connection")
      )
      if data.get("oauth_connection")
      else None,
      callback=mapServersDeploymentsGetOutputCallback.from_dict(data.get("callback"))
      if data.get("callback")
      else None,
      result=data.get("result"),
      metadata=data.get("metadata"),
      secret_id=data.get("secret_id"),
      server=mapServersDeploymentsGetOutputServer.from_dict(data.get("server"))
      if data.get("server")
      else None,
      config=mapServersDeploymentsGetOutputConfig.from_dict(data.get("config"))
      if data.get("config")
      else None,
      server_implementation=mapServersDeploymentsGetOutputServerImplementation.from_dict(
        data.get("server_implementation")
      )
      if data.get("server_implementation")
      else None,
      access=mapServersDeploymentsGetOutputAccess.from_dict(data.get("access"))
      if data.get("access")
      else None,
      created_at=parse_iso_datetime(data.get("created_at"))
      if data.get("created_at")
      else None,
      updated_at=parse_iso_datetime(data.get("updated_at"))
      if data.get("updated_at")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[ServersDeploymentsGetOutput, Dict[str, Any], None]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    # assume dataclass for generated models
    return dataclasses.asdict(value)



from dataclasses import dataclass
from typing import Any, Dict, List, Optional, Union
from datetime import datetime
from metorial.utils import parse_iso_datetime
import dataclasses


@dataclass
class DashboardInstanceMagicMcpSessionsListOutputPagination:
  has_more_before: bool
  has_more_after: bool


@dataclass
class DashboardInstanceMagicMcpSessionsListOutput:
  items: List[Dict[str, Any]]
  pagination: DashboardInstanceMagicMcpSessionsListOutputPagination


class mapDashboardInstanceMagicMcpSessionsListOutputPagination:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> DashboardInstanceMagicMcpSessionsListOutputPagination:
    return DashboardInstanceMagicMcpSessionsListOutputPagination(
      has_more_before=data.get("has_more_before"),
      has_more_after=data.get("has_more_after"),
    )

  @staticmethod
  def to_dict(
    value: Union[
      DashboardInstanceMagicMcpSessionsListOutputPagination, Dict[str, Any], None
    ]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapDashboardInstanceMagicMcpSessionsListOutput:
  @staticmethod
  def from_dict(data: Dict[str, Any]) -> DashboardInstanceMagicMcpSessionsListOutput:
    return DashboardInstanceMagicMcpSessionsListOutput(
      items=data.get("items", []),
      pagination=mapDashboardInstanceMagicMcpSessionsListOutputPagination.from_dict(
        data.get("pagination")
      )
      if data.get("pagination")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[DashboardInstanceMagicMcpSessionsListOutput, Dict[str, Any], None]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    # assume dataclass for generated models
    return dataclasses.asdict(value)


@dataclass
class DashboardInstanceMagicMcpSessionsListQuery:
  limit: Optional[float] = None
  after: Optional[str] = None
  before: Optional[str] = None
  cursor: Optional[str] = None
  order: Optional[str] = None
  magic_mcp_server_id: Optional[Union[str, List[str]]] = None


class mapDashboardInstanceMagicMcpSessionsListQuery:
  @staticmethod
  def from_dict(data: Dict[str, Any]) -> DashboardInstanceMagicMcpSessionsListQuery:
    return DashboardInstanceMagicMcpSessionsListQuery(
      limit=data.get("limit"),
      after=data.get("after"),
      before=data.get("before"),
      cursor=data.get("cursor"),
      order=data.get("order"),
      magic_mcp_server_id=data.get("magic_mcp_server_id"),
    )

  @staticmethod
  def to_dict(
    value: Union[DashboardInstanceMagicMcpSessionsListQuery, Dict[str, Any], None]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    # assume dataclass for generated models
    return dataclasses.asdict(value)



from dataclasses import dataclass
from typing import Any, Dict, List, Optional, Union
from datetime import datetime
import dataclasses

@dataclass
class DashboardInstanceMagicMcpServersListOutputItemsEndpoints:
    id: str
    alias: str
    url: str
@dataclass
class DashboardInstanceMagicMcpServersListOutputItems:
    object: str
    id: str
    status: str
    session_template_id: str
    endpoints: List[DashboardInstanceMagicMcpServersListOutputItemsEndpoints]
    metadata: Dict[str, Any]
    created_at: datetime
    updated_at: datetime
    name: Optional[str] = None
    description: Optional[str] = None
@dataclass
class DashboardInstanceMagicMcpServersListOutputPagination:
    has_more_before: bool
    has_more_after: bool
@dataclass
class DashboardInstanceMagicMcpServersListOutput:
    items: List[DashboardInstanceMagicMcpServersListOutputItems]
    pagination: DashboardInstanceMagicMcpServersListOutputPagination


class mapDashboardInstanceMagicMcpServersListOutputItemsEndpoints:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> DashboardInstanceMagicMcpServersListOutputItemsEndpoints:
        return DashboardInstanceMagicMcpServersListOutputItemsEndpoints(
        id=data.get('id'),
        alias=data.get('alias'),
        url=data.get('url')
        )

    @staticmethod
    def to_dict(value: Union[DashboardInstanceMagicMcpServersListOutputItemsEndpoints, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapDashboardInstanceMagicMcpServersListOutputItems:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> DashboardInstanceMagicMcpServersListOutputItems:
        return DashboardInstanceMagicMcpServersListOutputItems(
        object=data.get('object'),
        id=data.get('id'),
        status=data.get('status'),
        session_template_id=data.get('session_template_id'),
        endpoints=[mapDashboardInstanceMagicMcpServersListOutputItemsEndpoints.from_dict(item) for item in data.get('endpoints', []) if item],
        name=data.get('name'),
        description=data.get('description'),
        metadata=data.get('metadata'),
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None,
        updated_at=datetime.fromisoformat(data.get('updated_at').replace('Z', '+00:00')) if data.get('updated_at') else None
        )

    @staticmethod
    def to_dict(value: Union[DashboardInstanceMagicMcpServersListOutputItems, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapDashboardInstanceMagicMcpServersListOutputPagination:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> DashboardInstanceMagicMcpServersListOutputPagination:
        return DashboardInstanceMagicMcpServersListOutputPagination(
        has_more_before=data.get('has_more_before'),
        has_more_after=data.get('has_more_after')
        )

    @staticmethod
    def to_dict(value: Union[DashboardInstanceMagicMcpServersListOutputPagination, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapDashboardInstanceMagicMcpServersListOutput:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> DashboardInstanceMagicMcpServersListOutput:
        return DashboardInstanceMagicMcpServersListOutput(
        items=[mapDashboardInstanceMagicMcpServersListOutputItems.from_dict(item) for item in data.get('items', []) if item],
        pagination=mapDashboardInstanceMagicMcpServersListOutputPagination.from_dict(data.get('pagination')) if data.get('pagination') else None
        )

    @staticmethod
    def to_dict(value: Union[DashboardInstanceMagicMcpServersListOutput, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        # assume dataclass for generated models
        return dataclasses.asdict(value)

@dataclass
class DashboardInstanceMagicMcpServersListQuery:
    limit: Optional[float] = None
    after: Optional[str] = None
    before: Optional[str] = None
    cursor: Optional[str] = None
    order: Optional[str] = None
    status: Optional[Union[str, List[str]]] = None
    magic_mcp_group_id: Optional[Union[str, List[str]]] = None
    search: Optional[str] = None


class mapDashboardInstanceMagicMcpServersListQuery:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> DashboardInstanceMagicMcpServersListQuery:
        return DashboardInstanceMagicMcpServersListQuery(
        limit=data.get('limit'),
        after=data.get('after'),
        before=data.get('before'),
        cursor=data.get('cursor'),
        order=data.get('order'),
        status=data.get('status'),
        magic_mcp_group_id=data.get('magic_mcp_group_id'),
        search=data.get('search')
        )

    @staticmethod
    def to_dict(value: Union[DashboardInstanceMagicMcpServersListQuery, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        # assume dataclass for generated models
        return dataclasses.asdict(value)




from typing import Any, Dict, List, Optional, Union
from metorial_util_endpoint import BaseMetorialEndpoint, MetorialEndpointManager, MetorialRequest
from ..resources import mapDashboardInstanceMagicMcpTokensListOutput, DashboardInstanceMagicMcpTokensListOutput, mapDashboardInstanceMagicMcpTokensListQuery, DashboardInstanceMagicMcpTokensListQuery, mapDashboardInstanceMagicMcpTokensGetOutput, DashboardInstanceMagicMcpTokensGetOutput, mapDashboardInstanceMagicMcpTokensCreateOutput, DashboardInstanceMagicMcpTokensCreateOutput, mapDashboardInstanceMagicMcpTokensCreateBody, DashboardInstanceMagicMcpTokensCreateBody, mapDashboardInstanceMagicMcpTokensDeleteOutput, DashboardInstanceMagicMcpTokensDeleteOutput, mapDashboardInstanceMagicMcpTokensUpdateOutput, DashboardInstanceMagicMcpTokensUpdateOutput, mapDashboardInstanceMagicMcpTokensUpdateBody, DashboardInstanceMagicMcpTokensUpdateBody, mapDashboardInstanceMagicMcpTokensAddGroupsOutput, DashboardInstanceMagicMcpTokensAddGroupsOutput, mapDashboardInstanceMagicMcpTokensAddGroupsBody, DashboardInstanceMagicMcpTokensAddGroupsBody, mapDashboardInstanceMagicMcpTokensRemoveGroupsOutput, DashboardInstanceMagicMcpTokensRemoveGroupsOutput, mapDashboardInstanceMagicMcpTokensRemoveGroupsBody, DashboardInstanceMagicMcpTokensRemoveGroupsBody

class MetorialMagicMcpTokensEndpoint(BaseMetorialEndpoint):
    """Magic MCP tokens authorize access to Magic MCP servers via the /magic connection API."""

    def __init__(self, config: MetorialEndpointManager):
        super().__init__(config)

    def list(self, *, limit: Optional[float] = None, after: Optional[str] = None, before: Optional[str] = None, cursor: Optional[str] = None, order: Optional[str] = None, status: Optional[Union[str, List[str]]] = None, magic_mcp_group_id: Optional[Union[str, List[str]]] = None) -> DashboardInstanceMagicMcpTokensListOutput:
        """
    List magic MCP tokens
    Returns a paginated list of magic MCP tokens.

    :param limit: Optional[float] (optional)
    :param after: Optional[str] (optional)
    :param before: Optional[str] (optional)
    :param cursor: Optional[str] (optional)
    :param order: Optional[str] (optional)
    :param status: Optional[Union[str, List[str]]] (optional)
    :param magic_mcp_group_id: Optional[Union[str, List[str]]] (optional)
    :return: DashboardInstanceMagicMcpTokensListOutput
    """
        # Build query parameters from keyword arguments
        query_dict = {}
        if limit is not None:
            query_dict["limit"] = limit
        if after is not None:
            query_dict["after"] = after
        if before is not None:
            query_dict["before"] = before
        if cursor is not None:
            query_dict["cursor"] = cursor
        if order is not None:
            query_dict["order"] = order
        if status is not None:
            query_dict["status"] = status
        if magic_mcp_group_id is not None:
            query_dict["magic_mcp_group_id"] = magic_mcp_group_id

        request = MetorialRequest(
            path=['magic-mcp-tokens'],
            query=query_dict
        )
        return self._get(request).transform(mapDashboardInstanceMagicMcpTokensListOutput.from_dict)

    def get(self, magic_mcp_token_id: str) -> DashboardInstanceMagicMcpTokensGetOutput:
        """
    Get magic MCP token
    Retrieves a specific magic MCP token.

    :param magic_mcp_token_id: str
    :return: DashboardInstanceMagicMcpTokensGetOutput
    """
        request = MetorialRequest(
            path=['magic-mcp-tokens', magic_mcp_token_id]
        )
        return self._get(request).transform(mapDashboardInstanceMagicMcpTokensGetOutput.from_dict)

    def create(self, *, name: str, description: Optional[str] = None, metadata: Optional[Dict[str, Any]] = None, group_ids: Optional[List[str]] = None) -> DashboardInstanceMagicMcpTokensCreateOutput:
        """
    Create magic MCP token
    Creates a new magic MCP token.

    :param name: str
    :param description: Optional[str] (optional)
    :param metadata: Optional[Dict[str, Any]] (optional)
    :param group_ids: Optional[List[str]] (optional)
    :return: DashboardInstanceMagicMcpTokensCreateOutput
    """
        # Build body parameters from keyword arguments
        body_dict = {}
        body_dict["name"] = name
        if description is not None:
            body_dict["description"] = description
        if metadata is not None:
            body_dict["metadata"] = metadata
        if group_ids is not None:
            body_dict["group_ids"] = group_ids

        request = MetorialRequest(
            path=['magic-mcp-tokens'],
            body=body_dict
        )
        return self._post(request).transform(mapDashboardInstanceMagicMcpTokensCreateOutput.from_dict)

    def delete(self, magic_mcp_token_id: str) -> DashboardInstanceMagicMcpTokensDeleteOutput:
        """
    Delete magic MCP token
    Deletes a magic MCP token.

    :param magic_mcp_token_id: str
    :return: DashboardInstanceMagicMcpTokensDeleteOutput
    """
        request = MetorialRequest(
            path=['magic-mcp-tokens', magic_mcp_token_id]
        )
        return self._delete(request).transform(mapDashboardInstanceMagicMcpTokensDeleteOutput.from_dict)

    def update(self, magic_mcp_token_id: str, *, name: Optional[str] = None, description: Optional[str] = None, metadata: Optional[Dict[str, Any]] = None) -> DashboardInstanceMagicMcpTokensUpdateOutput:
        """
    Update magic MCP token
    Updates a magic MCP token.

    :param magic_mcp_token_id: str
    :param name: Optional[str] (optional)
    :param description: Optional[str] (optional)
    :param metadata: Optional[Dict[str, Any]] (optional)
    :return: DashboardInstanceMagicMcpTokensUpdateOutput
    """
        # Build body parameters from keyword arguments
        body_dict = {}
        if name is not None:
            body_dict["name"] = name
        if description is not None:
            body_dict["description"] = description
        if metadata is not None:
            body_dict["metadata"] = metadata

        request = MetorialRequest(
            path=['magic-mcp-tokens', magic_mcp_token_id],
            body=body_dict
        )
        return self._patch(request).transform(mapDashboardInstanceMagicMcpTokensUpdateOutput.from_dict)

    def add_groups(self, magic_mcp_token_id: str, *, magic_mcp_group_ids: List[str]) -> DashboardInstanceMagicMcpTokensAddGroupsOutput:
        """
    Add magic MCP groups to token
    Adds groups to a magic MCP token.

    :param magic_mcp_token_id: str
    :param magic_mcp_group_ids: List[str]
    :return: DashboardInstanceMagicMcpTokensAddGroupsOutput
    """
        # Build body parameters from keyword arguments
        body_dict = {}
        body_dict["magic_mcp_group_ids"] = magic_mcp_group_ids

        request = MetorialRequest(
            path=['magic-mcp-tokens', magic_mcp_token_id, 'add-groups'],
            body=body_dict
        )
        return self._post(request).transform(mapDashboardInstanceMagicMcpTokensAddGroupsOutput.from_dict)

    def remove_groups(self, magic_mcp_token_id: str, *, magic_mcp_group_ids: List[str]) -> DashboardInstanceMagicMcpTokensRemoveGroupsOutput:
        """
    Remove magic MCP groups from token
    Removes groups from a magic MCP token.

    :param magic_mcp_token_id: str
    :param magic_mcp_group_ids: List[str]
    :return: DashboardInstanceMagicMcpTokensRemoveGroupsOutput
    """
        # Build body parameters from keyword arguments
        body_dict = {}
        body_dict["magic_mcp_group_ids"] = magic_mcp_group_ids

        request = MetorialRequest(
            path=['magic-mcp-tokens', magic_mcp_token_id, 'remove-groups'],
            body=body_dict
        )
        return self._post(request).transform(mapDashboardInstanceMagicMcpTokensRemoveGroupsOutput.from_dict)


from dataclasses import dataclass
from typing import Any, Dict, List, Optional, Union
from datetime import datetime
import dataclasses

@dataclass
class DashboardInstanceCustomProvidersVersionsCreateOutputConfigSchema:
    type: str
    schema: Dict[str, Any]
@dataclass
class DashboardInstanceCustomProvidersVersionsCreateOutputConfig:
    object: str
    schema: DashboardInstanceCustomProvidersVersionsCreateOutputConfigSchema
    transformer: str
@dataclass
class DashboardInstanceCustomProvidersVersionsCreateOutputDeploymentCommit:
    object: str
    id: str
    type: str
    created_at: datetime
    message: Optional[str] = None
@dataclass
class DashboardInstanceCustomProvidersVersionsCreateOutputDeploymentImmutableBucketScmRepoLinkRepositoryProvider:
    object: str
    type: str
    id: str
    name: str
    owner: str
@dataclass
class DashboardInstanceCustomProvidersVersionsCreateOutputDeploymentImmutableBucketScmRepoLinkRepository:
    object: str
    id: str
    provider: DashboardInstanceCustomProvidersVersionsCreateOutputDeploymentImmutableBucketScmRepoLinkRepositoryProvider
    url: str
    is_private: bool
    default_branch: str
    created_at: datetime
@dataclass
class DashboardInstanceCustomProvidersVersionsCreateOutputDeploymentImmutableBucketScmRepoLink:
    object: str
    is_linked: str
    repository: DashboardInstanceCustomProvidersVersionsCreateOutputDeploymentImmutableBucketScmRepoLinkRepository
    path: Optional[str] = None
@dataclass
class DashboardInstanceCustomProvidersVersionsCreateOutputDeploymentImmutableBucket:
    object: str
    id: str
    is_immutable: bool
    is_read_only: bool
    created_at: datetime
    scm_repo_link: Optional[DashboardInstanceCustomProvidersVersionsCreateOutputDeploymentImmutableBucketScmRepoLink] = None
@dataclass
class DashboardInstanceCustomProvidersVersionsCreateOutputDeploymentActor:
    object: str
    id: str
    type: str
    identifier: str
    name: str
    created_at: datetime
    organization_actor_id: Optional[str] = None
@dataclass
class DashboardInstanceCustomProvidersVersionsCreateOutputDeploymentScmPushActor:
    object: str
    id: str
    external_id: Optional[str] = None
    name: Optional[str] = None
    email: Optional[str] = None
@dataclass
class DashboardInstanceCustomProvidersVersionsCreateOutputDeploymentScmPushCommit:
    object: str
    id: str
    sha: str
    branch: str
    created_at: datetime
    message: Optional[str] = None
@dataclass
class DashboardInstanceCustomProvidersVersionsCreateOutputDeploymentScmPushRepositoryProvider:
    object: str
    type: str
    id: str
    name: str
    owner: str
@dataclass
class DashboardInstanceCustomProvidersVersionsCreateOutputDeploymentScmPushRepository:
    object: str
    id: str
    provider: DashboardInstanceCustomProvidersVersionsCreateOutputDeploymentScmPushRepositoryProvider
    url: str
    is_private: bool
    default_branch: str
    created_at: datetime
@dataclass
class DashboardInstanceCustomProvidersVersionsCreateOutputDeploymentScmPush:
    object: str
    id: str
    actor: DashboardInstanceCustomProvidersVersionsCreateOutputDeploymentScmPushActor
    commit: DashboardInstanceCustomProvidersVersionsCreateOutputDeploymentScmPushCommit
    repository: DashboardInstanceCustomProvidersVersionsCreateOutputDeploymentScmPushRepository
    created_at: datetime
@dataclass
class DashboardInstanceCustomProvidersVersionsCreateOutputDeployment:
    object: str
    id: str
    status: str
    trigger: str
    custom_provider_id: str
    actor: DashboardInstanceCustomProvidersVersionsCreateOutputDeploymentActor
    created_at: datetime
    updated_at: datetime
    provider_id: Optional[str] = None
    custom_provider_version_id: Optional[str] = None
    commit: Optional[DashboardInstanceCustomProvidersVersionsCreateOutputDeploymentCommit] = None
    immutable_bucket: Optional[DashboardInstanceCustomProvidersVersionsCreateOutputDeploymentImmutableBucket] = None
    scm_push: Optional[DashboardInstanceCustomProvidersVersionsCreateOutputDeploymentScmPush] = None
@dataclass
class DashboardInstanceCustomProvidersVersionsCreateOutputEnvironmentsEnvironment:
    object: str
    id: str
    custom_provider_id: str
    instance_id: str
    created_at: datetime
    updated_at: datetime
    provider_id: Optional[str] = None
    current_provider_version_id: Optional[str] = None
@dataclass
class DashboardInstanceCustomProvidersVersionsCreateOutputEnvironments:
    object: str
    id: str
    is_current_version_for_environment: bool
    environment: DashboardInstanceCustomProvidersVersionsCreateOutputEnvironmentsEnvironment
@dataclass
class DashboardInstanceCustomProvidersVersionsCreateOutputActor:
    object: str
    id: str
    type: str
    identifier: str
    name: str
    created_at: datetime
    organization_actor_id: Optional[str] = None
@dataclass
class DashboardInstanceCustomProvidersVersionsCreateOutputContainerImage:
    container_registry: str
    container_image_tag: str
    container_image: str
@dataclass
class DashboardInstanceCustomProvidersVersionsCreateOutputRemoteMcpServer:
    url: str
    transport: str
@dataclass
class DashboardInstanceCustomProvidersVersionsCreateOutput:
    object: str
    id: str
    status: str
    index: float
    identifier: str
    deployment: DashboardInstanceCustomProvidersVersionsCreateOutputDeployment
    environments: List[DashboardInstanceCustomProvidersVersionsCreateOutputEnvironments]
    custom_provider_id: str
    actor: DashboardInstanceCustomProvidersVersionsCreateOutputActor
    created_at: datetime
    updated_at: datetime
    config: Optional[DashboardInstanceCustomProvidersVersionsCreateOutputConfig] = None
    provider_id: Optional[str] = None
    container_image: Optional[DashboardInstanceCustomProvidersVersionsCreateOutputContainerImage] = None
    remote_mcp_server: Optional[DashboardInstanceCustomProvidersVersionsCreateOutputRemoteMcpServer] = None


class mapDashboardInstanceCustomProvidersVersionsCreateOutputConfigSchema:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> DashboardInstanceCustomProvidersVersionsCreateOutputConfigSchema:
        return DashboardInstanceCustomProvidersVersionsCreateOutputConfigSchema(
        type=data.get('type'),
        schema=data.get('schema')
        )

    @staticmethod
    def to_dict(value: Union[DashboardInstanceCustomProvidersVersionsCreateOutputConfigSchema, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapDashboardInstanceCustomProvidersVersionsCreateOutputConfig:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> DashboardInstanceCustomProvidersVersionsCreateOutputConfig:
        return DashboardInstanceCustomProvidersVersionsCreateOutputConfig(
        object=data.get('object'),
        schema=mapDashboardInstanceCustomProvidersVersionsCreateOutputConfigSchema.from_dict(data.get('schema')) if data.get('schema') else None,
        transformer=data.get('transformer')
        )

    @staticmethod
    def to_dict(value: Union[DashboardInstanceCustomProvidersVersionsCreateOutputConfig, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapDashboardInstanceCustomProvidersVersionsCreateOutputDeploymentCommit:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> DashboardInstanceCustomProvidersVersionsCreateOutputDeploymentCommit:
        return DashboardInstanceCustomProvidersVersionsCreateOutputDeploymentCommit(
        object=data.get('object'),
        id=data.get('id'),
        type=data.get('type'),
        message=data.get('message'),
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None
        )

    @staticmethod
    def to_dict(value: Union[DashboardInstanceCustomProvidersVersionsCreateOutputDeploymentCommit, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapDashboardInstanceCustomProvidersVersionsCreateOutputDeploymentImmutableBucketScmRepoLinkRepositoryProvider:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> DashboardInstanceCustomProvidersVersionsCreateOutputDeploymentImmutableBucketScmRepoLinkRepositoryProvider:
        return DashboardInstanceCustomProvidersVersionsCreateOutputDeploymentImmutableBucketScmRepoLinkRepositoryProvider(
        object=data.get('object'),
        type=data.get('type'),
        id=data.get('id'),
        name=data.get('name'),
        owner=data.get('owner')
        )

    @staticmethod
    def to_dict(value: Union[DashboardInstanceCustomProvidersVersionsCreateOutputDeploymentImmutableBucketScmRepoLinkRepositoryProvider, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapDashboardInstanceCustomProvidersVersionsCreateOutputDeploymentImmutableBucketScmRepoLinkRepository:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> DashboardInstanceCustomProvidersVersionsCreateOutputDeploymentImmutableBucketScmRepoLinkRepository:
        return DashboardInstanceCustomProvidersVersionsCreateOutputDeploymentImmutableBucketScmRepoLinkRepository(
        object=data.get('object'),
        id=data.get('id'),
        provider=mapDashboardInstanceCustomProvidersVersionsCreateOutputDeploymentImmutableBucketScmRepoLinkRepositoryProvider.from_dict(data.get('provider')) if data.get('provider') else None,
        url=data.get('url'),
        is_private=data.get('is_private'),
        default_branch=data.get('default_branch'),
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None
        )

    @staticmethod
    def to_dict(value: Union[DashboardInstanceCustomProvidersVersionsCreateOutputDeploymentImmutableBucketScmRepoLinkRepository, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapDashboardInstanceCustomProvidersVersionsCreateOutputDeploymentImmutableBucketScmRepoLink:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> DashboardInstanceCustomProvidersVersionsCreateOutputDeploymentImmutableBucketScmRepoLink:
        return DashboardInstanceCustomProvidersVersionsCreateOutputDeploymentImmutableBucketScmRepoLink(
        object=data.get('object'),
        is_linked=data.get('is_linked'),
        path=data.get('path'),
        repository=mapDashboardInstanceCustomProvidersVersionsCreateOutputDeploymentImmutableBucketScmRepoLinkRepository.from_dict(data.get('repository')) if data.get('repository') else None
        )

    @staticmethod
    def to_dict(value: Union[DashboardInstanceCustomProvidersVersionsCreateOutputDeploymentImmutableBucketScmRepoLink, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapDashboardInstanceCustomProvidersVersionsCreateOutputDeploymentImmutableBucket:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> DashboardInstanceCustomProvidersVersionsCreateOutputDeploymentImmutableBucket:
        return DashboardInstanceCustomProvidersVersionsCreateOutputDeploymentImmutableBucket(
        object=data.get('object'),
        id=data.get('id'),
        is_immutable=data.get('is_immutable'),
        is_read_only=data.get('is_read_only'),
        scm_repo_link=mapDashboardInstanceCustomProvidersVersionsCreateOutputDeploymentImmutableBucketScmRepoLink.from_dict(data.get('scm_repo_link')) if data.get('scm_repo_link') else None,
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None
        )

    @staticmethod
    def to_dict(value: Union[DashboardInstanceCustomProvidersVersionsCreateOutputDeploymentImmutableBucket, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapDashboardInstanceCustomProvidersVersionsCreateOutputDeploymentActor:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> DashboardInstanceCustomProvidersVersionsCreateOutputDeploymentActor:
        return DashboardInstanceCustomProvidersVersionsCreateOutputDeploymentActor(
        object=data.get('object'),
        id=data.get('id'),
        type=data.get('type'),
        identifier=data.get('identifier'),
        name=data.get('name'),
        organization_actor_id=data.get('organization_actor_id'),
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None
        )

    @staticmethod
    def to_dict(value: Union[DashboardInstanceCustomProvidersVersionsCreateOutputDeploymentActor, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapDashboardInstanceCustomProvidersVersionsCreateOutputDeploymentScmPushActor:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> DashboardInstanceCustomProvidersVersionsCreateOutputDeploymentScmPushActor:
        return DashboardInstanceCustomProvidersVersionsCreateOutputDeploymentScmPushActor(
        object=data.get('object'),
        id=data.get('id'),
        external_id=data.get('external_id'),
        name=data.get('name'),
        email=data.get('email')
        )

    @staticmethod
    def to_dict(value: Union[DashboardInstanceCustomProvidersVersionsCreateOutputDeploymentScmPushActor, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapDashboardInstanceCustomProvidersVersionsCreateOutputDeploymentScmPushCommit:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> DashboardInstanceCustomProvidersVersionsCreateOutputDeploymentScmPushCommit:
        return DashboardInstanceCustomProvidersVersionsCreateOutputDeploymentScmPushCommit(
        object=data.get('object'),
        id=data.get('id'),
        sha=data.get('sha'),
        branch=data.get('branch'),
        message=data.get('message'),
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None
        )

    @staticmethod
    def to_dict(value: Union[DashboardInstanceCustomProvidersVersionsCreateOutputDeploymentScmPushCommit, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapDashboardInstanceCustomProvidersVersionsCreateOutputDeploymentScmPushRepositoryProvider:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> DashboardInstanceCustomProvidersVersionsCreateOutputDeploymentScmPushRepositoryProvider:
        return DashboardInstanceCustomProvidersVersionsCreateOutputDeploymentScmPushRepositoryProvider(
        object=data.get('object'),
        type=data.get('type'),
        id=data.get('id'),
        name=data.get('name'),
        owner=data.get('owner')
        )

    @staticmethod
    def to_dict(value: Union[DashboardInstanceCustomProvidersVersionsCreateOutputDeploymentScmPushRepositoryProvider, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapDashboardInstanceCustomProvidersVersionsCreateOutputDeploymentScmPushRepository:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> DashboardInstanceCustomProvidersVersionsCreateOutputDeploymentScmPushRepository:
        return DashboardInstanceCustomProvidersVersionsCreateOutputDeploymentScmPushRepository(
        object=data.get('object'),
        id=data.get('id'),
        provider=mapDashboardInstanceCustomProvidersVersionsCreateOutputDeploymentScmPushRepositoryProvider.from_dict(data.get('provider')) if data.get('provider') else None,
        url=data.get('url'),
        is_private=data.get('is_private'),
        default_branch=data.get('default_branch'),
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None
        )

    @staticmethod
    def to_dict(value: Union[DashboardInstanceCustomProvidersVersionsCreateOutputDeploymentScmPushRepository, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapDashboardInstanceCustomProvidersVersionsCreateOutputDeploymentScmPush:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> DashboardInstanceCustomProvidersVersionsCreateOutputDeploymentScmPush:
        return DashboardInstanceCustomProvidersVersionsCreateOutputDeploymentScmPush(
        object=data.get('object'),
        id=data.get('id'),
        actor=mapDashboardInstanceCustomProvidersVersionsCreateOutputDeploymentScmPushActor.from_dict(data.get('actor')) if data.get('actor') else None,
        commit=mapDashboardInstanceCustomProvidersVersionsCreateOutputDeploymentScmPushCommit.from_dict(data.get('commit')) if data.get('commit') else None,
        repository=mapDashboardInstanceCustomProvidersVersionsCreateOutputDeploymentScmPushRepository.from_dict(data.get('repository')) if data.get('repository') else None,
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None
        )

    @staticmethod
    def to_dict(value: Union[DashboardInstanceCustomProvidersVersionsCreateOutputDeploymentScmPush, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapDashboardInstanceCustomProvidersVersionsCreateOutputDeployment:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> DashboardInstanceCustomProvidersVersionsCreateOutputDeployment:
        return DashboardInstanceCustomProvidersVersionsCreateOutputDeployment(
        object=data.get('object'),
        id=data.get('id'),
        status=data.get('status'),
        trigger=data.get('trigger'),
        custom_provider_id=data.get('custom_provider_id'),
        provider_id=data.get('provider_id'),
        custom_provider_version_id=data.get('custom_provider_version_id'),
        commit=mapDashboardInstanceCustomProvidersVersionsCreateOutputDeploymentCommit.from_dict(data.get('commit')) if data.get('commit') else None,
        immutable_bucket=mapDashboardInstanceCustomProvidersVersionsCreateOutputDeploymentImmutableBucket.from_dict(data.get('immutable_bucket')) if data.get('immutable_bucket') else None,
        actor=mapDashboardInstanceCustomProvidersVersionsCreateOutputDeploymentActor.from_dict(data.get('actor')) if data.get('actor') else None,
        scm_push=mapDashboardInstanceCustomProvidersVersionsCreateOutputDeploymentScmPush.from_dict(data.get('scm_push')) if data.get('scm_push') else None,
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None,
        updated_at=datetime.fromisoformat(data.get('updated_at').replace('Z', '+00:00')) if data.get('updated_at') else None
        )

    @staticmethod
    def to_dict(value: Union[DashboardInstanceCustomProvidersVersionsCreateOutputDeployment, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapDashboardInstanceCustomProvidersVersionsCreateOutputEnvironmentsEnvironment:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> DashboardInstanceCustomProvidersVersionsCreateOutputEnvironmentsEnvironment:
        return DashboardInstanceCustomProvidersVersionsCreateOutputEnvironmentsEnvironment(
        object=data.get('object'),
        id=data.get('id'),
        custom_provider_id=data.get('custom_provider_id'),
        provider_id=data.get('provider_id'),
        current_provider_version_id=data.get('current_provider_version_id'),
        instance_id=data.get('instance_id'),
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None,
        updated_at=datetime.fromisoformat(data.get('updated_at').replace('Z', '+00:00')) if data.get('updated_at') else None
        )

    @staticmethod
    def to_dict(value: Union[DashboardInstanceCustomProvidersVersionsCreateOutputEnvironmentsEnvironment, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapDashboardInstanceCustomProvidersVersionsCreateOutputEnvironments:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> DashboardInstanceCustomProvidersVersionsCreateOutputEnvironments:
        return DashboardInstanceCustomProvidersVersionsCreateOutputEnvironments(
        object=data.get('object'),
        id=data.get('id'),
        is_current_version_for_environment=data.get('is_current_version_for_environment'),
        environment=mapDashboardInstanceCustomProvidersVersionsCreateOutputEnvironmentsEnvironment.from_dict(data.get('environment')) if data.get('environment') else None
        )

    @staticmethod
    def to_dict(value: Union[DashboardInstanceCustomProvidersVersionsCreateOutputEnvironments, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapDashboardInstanceCustomProvidersVersionsCreateOutputActor:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> DashboardInstanceCustomProvidersVersionsCreateOutputActor:
        return DashboardInstanceCustomProvidersVersionsCreateOutputActor(
        object=data.get('object'),
        id=data.get('id'),
        type=data.get('type'),
        identifier=data.get('identifier'),
        name=data.get('name'),
        organization_actor_id=data.get('organization_actor_id'),
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None
        )

    @staticmethod
    def to_dict(value: Union[DashboardInstanceCustomProvidersVersionsCreateOutputActor, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapDashboardInstanceCustomProvidersVersionsCreateOutputContainerImage:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> DashboardInstanceCustomProvidersVersionsCreateOutputContainerImage:
        return DashboardInstanceCustomProvidersVersionsCreateOutputContainerImage(
        container_registry=data.get('container_registry'),
        container_image_tag=data.get('container_image_tag'),
        container_image=data.get('container_image')
        )

    @staticmethod
    def to_dict(value: Union[DashboardInstanceCustomProvidersVersionsCreateOutputContainerImage, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapDashboardInstanceCustomProvidersVersionsCreateOutputRemoteMcpServer:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> DashboardInstanceCustomProvidersVersionsCreateOutputRemoteMcpServer:
        return DashboardInstanceCustomProvidersVersionsCreateOutputRemoteMcpServer(
        url=data.get('url'),
        transport=data.get('transport')
        )

    @staticmethod
    def to_dict(value: Union[DashboardInstanceCustomProvidersVersionsCreateOutputRemoteMcpServer, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapDashboardInstanceCustomProvidersVersionsCreateOutput:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> DashboardInstanceCustomProvidersVersionsCreateOutput:
        return DashboardInstanceCustomProvidersVersionsCreateOutput(
        object=data.get('object'),
        id=data.get('id'),
        status=data.get('status'),
        config=mapDashboardInstanceCustomProvidersVersionsCreateOutputConfig.from_dict(data.get('config')) if data.get('config') else None,
        index=data.get('index'),
        identifier=data.get('identifier'),
        deployment=mapDashboardInstanceCustomProvidersVersionsCreateOutputDeployment.from_dict(data.get('deployment')) if data.get('deployment') else None,
        environments=[mapDashboardInstanceCustomProvidersVersionsCreateOutputEnvironments.from_dict(item) for item in data.get('environments', []) if item],
        custom_provider_id=data.get('custom_provider_id'),
        provider_id=data.get('provider_id'),
        actor=mapDashboardInstanceCustomProvidersVersionsCreateOutputActor.from_dict(data.get('actor')) if data.get('actor') else None,
        container_image=mapDashboardInstanceCustomProvidersVersionsCreateOutputContainerImage.from_dict(data.get('container_image')) if data.get('container_image') else None,
        remote_mcp_server=mapDashboardInstanceCustomProvidersVersionsCreateOutputRemoteMcpServer.from_dict(data.get('remote_mcp_server')) if data.get('remote_mcp_server') else None,
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None,
        updated_at=datetime.fromisoformat(data.get('updated_at').replace('Z', '+00:00')) if data.get('updated_at') else None
        )

    @staticmethod
    def to_dict(value: Union[DashboardInstanceCustomProvidersVersionsCreateOutput, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        # assume dataclass for generated models
        return dataclasses.asdict(value)

@dataclass
class DashboardInstanceCustomProvidersVersionsCreateBodyConfig:
    schema: Dict[str, Any]
    transformer: str
@dataclass
class DashboardInstanceCustomProvidersVersionsCreateBody:
    custom_provider_id: str
    from_: Dict[str, Any]
    config: Optional[DashboardInstanceCustomProvidersVersionsCreateBodyConfig] = None


class mapDashboardInstanceCustomProvidersVersionsCreateBodyConfig:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> DashboardInstanceCustomProvidersVersionsCreateBodyConfig:
        return DashboardInstanceCustomProvidersVersionsCreateBodyConfig(
        schema=data.get('schema'),
        transformer=data.get('transformer')
        )

    @staticmethod
    def to_dict(value: Union[DashboardInstanceCustomProvidersVersionsCreateBodyConfig, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapDashboardInstanceCustomProvidersVersionsCreateBody:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> DashboardInstanceCustomProvidersVersionsCreateBody:
        return DashboardInstanceCustomProvidersVersionsCreateBody(
        custom_provider_id=data.get('custom_provider_id'),
        from_=data.get('from'),
        config=mapDashboardInstanceCustomProvidersVersionsCreateBodyConfig.from_dict(data.get('config')) if data.get('config') else None
        )

    @staticmethod
    def to_dict(value: Union[DashboardInstanceCustomProvidersVersionsCreateBody, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        # assume dataclass for generated models
        return dataclasses.asdict(value)




"""Autogen integration for Metorial."""

from __future__ import annotations

import json
from collections.abc import Callable
from typing import TYPE_CHECKING, Any

if TYPE_CHECKING:
  from metorial._client import ProviderSession

try:
  from autogen_core.tools import FunctionTool
except ImportError:
  FunctionTool = None


def create_autogen_tools(session: ProviderSession) -> list[Any]:
  """
  Convert Metorial session tools to Autogen FunctionTool objects.

  Args:
      session: An active Metorial ProviderSession

  Returns:
      List of Autogen FunctionTool objects

  Example:
      ```python
      from autogen_agentchat.agents import AssistantAgent
      from autogen_ext.models.anthropic import AnthropicChatCompletionClient
      from metorial import Metorial
      from metorial.integrations.autogen import create_autogen_tools

      metorial = Metorial(api_key="...")

      async with metorial.provider_session(
          provider="anthropic",
          providers=[{"session_template_id": "deployment-id"}],
      ) as session:
          tools = create_autogen_tools(session)

          model_client = AnthropicChatCompletionClient(model="claude-sonnet-4-20250514")
          assistant = AssistantAgent(
              name="assistant",
              model_client=model_client,
              tools=tools,
          )
      ```
  """
  if FunctionTool is None:
    raise ImportError(
      "autogen-core is required for Autogen integration. "
      "Install it with: pip install autogen-agentchat autogen-ext"
    )

  tools = []
  tool_manager = session.tool_manager

  if tool_manager is None:
    return tools

  for tool in tool_manager.get_tools():
    tool_name = tool.name
    tool_description = tool.description or f"Tool: {tool_name}"

    schema = tool.get_parameters_as("json-schema") or {}
    properties = schema.get("properties", {})
    required_params = set(schema.get("required", []))

    # Build parameter list for the function signature
    type_map = {
      "string": str,
      "integer": int,
      "number": float,
      "boolean": bool,
      "array": list,
      "object": dict,
    }

    # Sort: required params first, optional params after
    required_props = [(n, s) for n, s in properties.items() if n in required_params]
    optional_props = [(n, s) for n, s in properties.items() if n not in required_params]

    params = []
    param_annotations = {}
    for prop_name, prop_schema in required_props + optional_props:
      json_type = prop_schema.get("type", "string")
      py_type = type_map.get(json_type, str)
      if prop_name not in required_params:
        py_type = py_type | None
      param_annotations[prop_name] = py_type
      if prop_name in required_params:
        params.append(f"{prop_name}: __annotations__['{prop_name}']")
      else:
        params.append(f"{prop_name}: __annotations__['{prop_name}'] = None")
    param_annotations["return"] = str

    func_name = tool_name.replace("-", "_")
    params_str = ", ".join(params) if params else ""

    # Dynamically create a function with explicit parameters
    # so autogen's FunctionTool can inspect the signature
    func_code = f"async def {func_name}({params_str}) -> str:\n"
    func_code += f"  '''{tool_description}'''\n"
    func_code += (
      f"  _kwargs = {{{', '.join(repr(p) + ': ' + p for p in properties)}}}\n"
    )
    func_code += "  _kwargs = {k: v for k, v in _kwargs.items() if v is not None}\n"
    func_code += "  return await _executor(_tool_name, _kwargs)\n"

    async def _make_executor(name: str):
      async def _executor(tool_name_inner: str, kwargs: dict) -> str:
        try:
          result = await tool_manager.execute_tool(tool_name_inner, kwargs)
          if hasattr(result, "model_dump"):
            result = result.model_dump()
          return json.dumps(result, ensure_ascii=False, default=str)
        except Exception as e:
          return json.dumps({"error": str(e)}, ensure_ascii=False)

      return _executor

    # We can't await here, so create the executor synchronously
    def _make_executor_sync(name: str):
      async def _executor(tool_name_inner: str, kwargs: dict) -> str:
        try:
          result = await tool_manager.execute_tool(tool_name_inner, kwargs)
          if hasattr(result, "model_dump"):
            result = result.model_dump()
          return json.dumps(result, ensure_ascii=False, default=str)
        except Exception as e:
          return json.dumps({"error": str(e)}, ensure_ascii=False)

      return _executor

    executor = _make_executor_sync(tool_name)
    local_ns: dict[str, Any] = {
      "_executor": executor,
      "_tool_name": tool_name,
      "__annotations__": param_annotations,
    }
    exec(func_code, local_ns)  # noqa: S102
    fn = local_ns[func_name]

    autogen_tool = FunctionTool(fn, description=tool_description, name=func_name)
    tools.append(autogen_tool)

  return tools


def get_autogen_tool_executor(session: ProviderSession) -> dict[str, Callable]:
  """
  Get a function map for legacy Autogen tool execution.

  .. deprecated::
      Use create_autogen_tools() with the new Autogen API instead.

  Args:
      session: An active Metorial ProviderSession

  Returns:
      Dictionary mapping tool names to executor functions
  """
  import asyncio

  metorial_tools = session.get_tools()
  function_map: dict[str, Callable] = {}

  for tool in metorial_tools:
    # Handle OpenAI-style format (type: function, function: {name, ...})
    if "function" in tool:
      tool_name = tool["function"].get("name", "")
    else:
      tool_name = tool.get("name", "")

    def make_executor(name: str) -> Callable:
      def executor(**kwargs: Any) -> str:
        async def call():
          result = await session.call_tool(name, kwargs)
          if isinstance(result, dict):
            content = result.get("content", [])
            if isinstance(content, list):
              texts = []
              for item in content:
                if isinstance(item, dict) and "text" in item:
                  texts.append(item["text"])
              return "\n".join(texts) if texts else str(result)
            return str(content)
          return str(result)

        try:
          asyncio.get_running_loop()
          import concurrent.futures

          with concurrent.futures.ThreadPoolExecutor() as pool:
            future = pool.submit(asyncio.run, call())
            return future.result()
        except RuntimeError:
          return asyncio.run(call())

      return executor

    function_map[tool_name] = make_executor(tool_name)

  return function_map



from dataclasses import dataclass
from typing import Any, Dict, List, Optional, Union
from datetime import datetime
import dataclasses

@dataclass
class ManagementInstanceSessionsProvidersCreateOutputUsage:
    total_productive_client_message_count: float
    total_productive_provider_message_count: float
@dataclass
class ManagementInstanceSessionsProvidersCreateOutputDeployment:
    object: str
    id: str
    is_default: bool
    provider_id: str
    created_at: datetime
    updated_at: datetime
    name: Optional[str] = None
    description: Optional[str] = None
    metadata: Optional[Dict[str, Any]] = None
@dataclass
class ManagementInstanceSessionsProvidersCreateOutputConfig:
    object: str
    id: str
    is_default: bool
    provider_id: str
    created_at: datetime
    updated_at: datetime
    name: Optional[str] = None
    description: Optional[str] = None
    metadata: Optional[Dict[str, Any]] = None
@dataclass
class ManagementInstanceSessionsProvidersCreateOutputAuthConfig:
    object: str
    id: str
@dataclass
class ManagementInstanceSessionsProvidersCreateOutput:
    object: str
    id: str
    status: str
    usage: ManagementInstanceSessionsProvidersCreateOutputUsage
    tool_filter: Dict[str, Any]
    provider_id: str
    session_id: str
    deployment: ManagementInstanceSessionsProvidersCreateOutputDeployment
    config: ManagementInstanceSessionsProvidersCreateOutputConfig
    created_at: datetime
    updated_at: datetime
    from_template_id: Optional[str] = None
    from_template_provider_id: Optional[str] = None
    auth_config: Optional[ManagementInstanceSessionsProvidersCreateOutputAuthConfig] = None


class mapManagementInstanceSessionsProvidersCreateOutputUsage:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceSessionsProvidersCreateOutputUsage:
        return ManagementInstanceSessionsProvidersCreateOutputUsage(
        total_productive_client_message_count=data.get('total_productive_client_message_count'),
        total_productive_provider_message_count=data.get('total_productive_provider_message_count')
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceSessionsProvidersCreateOutputUsage, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapManagementInstanceSessionsProvidersCreateOutputDeployment:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceSessionsProvidersCreateOutputDeployment:
        return ManagementInstanceSessionsProvidersCreateOutputDeployment(
        object=data.get('object'),
        id=data.get('id'),
        is_default=data.get('is_default'),
        name=data.get('name'),
        description=data.get('description'),
        metadata=data.get('metadata'),
        provider_id=data.get('provider_id'),
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None,
        updated_at=datetime.fromisoformat(data.get('updated_at').replace('Z', '+00:00')) if data.get('updated_at') else None
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceSessionsProvidersCreateOutputDeployment, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapManagementInstanceSessionsProvidersCreateOutputConfig:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceSessionsProvidersCreateOutputConfig:
        return ManagementInstanceSessionsProvidersCreateOutputConfig(
        object=data.get('object'),
        id=data.get('id'),
        is_default=data.get('is_default'),
        name=data.get('name'),
        description=data.get('description'),
        metadata=data.get('metadata'),
        provider_id=data.get('provider_id'),
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None,
        updated_at=datetime.fromisoformat(data.get('updated_at').replace('Z', '+00:00')) if data.get('updated_at') else None
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceSessionsProvidersCreateOutputConfig, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapManagementInstanceSessionsProvidersCreateOutputAuthConfig:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceSessionsProvidersCreateOutputAuthConfig:
        return ManagementInstanceSessionsProvidersCreateOutputAuthConfig(
        object=data.get('object'),
        id=data.get('id')
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceSessionsProvidersCreateOutputAuthConfig, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        return dataclasses.asdict(value)

class mapManagementInstanceSessionsProvidersCreateOutput:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceSessionsProvidersCreateOutput:
        return ManagementInstanceSessionsProvidersCreateOutput(
        object=data.get('object'),
        id=data.get('id'),
        status=data.get('status'),
        usage=mapManagementInstanceSessionsProvidersCreateOutputUsage.from_dict(data.get('usage')) if data.get('usage') else None,
        tool_filter=data.get('tool_filter'),
        provider_id=data.get('provider_id'),
        session_id=data.get('session_id'),
        from_template_id=data.get('from_template_id'),
        from_template_provider_id=data.get('from_template_provider_id'),
        deployment=mapManagementInstanceSessionsProvidersCreateOutputDeployment.from_dict(data.get('deployment')) if data.get('deployment') else None,
        config=mapManagementInstanceSessionsProvidersCreateOutputConfig.from_dict(data.get('config')) if data.get('config') else None,
        auth_config=mapManagementInstanceSessionsProvidersCreateOutputAuthConfig.from_dict(data.get('auth_config')) if data.get('auth_config') else None,
        created_at=datetime.fromisoformat(data.get('created_at').replace('Z', '+00:00')) if data.get('created_at') else None,
        updated_at=datetime.fromisoformat(data.get('updated_at').replace('Z', '+00:00')) if data.get('updated_at') else None
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceSessionsProvidersCreateOutput, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        # assume dataclass for generated models
        return dataclasses.asdict(value)

@dataclass
class ManagementInstanceSessionsProvidersCreateBody:
    session_id: str
    tool_filters: Optional[Union[Dict[str, Any], List[Dict[str, Any]]]] = None


class mapManagementInstanceSessionsProvidersCreateBody:
    @staticmethod
    def from_dict(data: Dict[str, Any]) -> ManagementInstanceSessionsProvidersCreateBody:
        return ManagementInstanceSessionsProvidersCreateBody(
        session_id=data.get('session_id'),
        tool_filters=data.get('tool_filters')
        )

    @staticmethod
    def to_dict(value: Union[ManagementInstanceSessionsProvidersCreateBody, Dict[str, Any], None]) -> Optional[Dict[str, Any]]:
        if value is None:
            return None
        if isinstance(value, dict):
            return value
        # assume dataclass for generated models
        return dataclasses.asdict(value)




from metorial_util_endpoint import BaseMetorialEndpoint, MetorialEndpointManager, MetorialRequest
from ..resources import mapDashboardOrganizationsOauthAppsClientSecretsCreateOutput, DashboardOrganizationsOauthAppsClientSecretsCreateOutput, mapDashboardOrganizationsOauthAppsClientSecretsDeleteOutput, DashboardOrganizationsOauthAppsClientSecretsDeleteOutput

class MetorialManagementOrganizationOauthAppsClientSecretsEndpoint(BaseMetorialEndpoint):
    """Create and manage OAuth applications for an organization"""

    def __init__(self, config: MetorialEndpointManager):
        super().__init__(config)

    def create(self, oauth_application_id: str) -> DashboardOrganizationsOauthAppsClientSecretsCreateOutput:
        """
    Create OAuth application client secret
    Creates a new client secret for an OAuth application.

    :param oauth_application_id: str
    :return: DashboardOrganizationsOauthAppsClientSecretsCreateOutput
    """
        request = MetorialRequest(
            path=['organization', 'oauth', 'apps', oauth_application_id, 'client-secrets']
        )
        return self._post(request).transform(mapDashboardOrganizationsOauthAppsClientSecretsCreateOutput.from_dict)

    def delete(self, oauth_application_id: str, oauth_application_client_secret_id: str) -> DashboardOrganizationsOauthAppsClientSecretsDeleteOutput:
        """
    Delete OAuth application client secret
    Deletes a client secret from an OAuth application.

    :param oauth_application_id: str
    :param oauth_application_client_secret_id: str
    :return: DashboardOrganizationsOauthAppsClientSecretsDeleteOutput
    """
        request = MetorialRequest(
            path=['organization', 'oauth', 'apps', oauth_application_id, 'client-secrets', oauth_application_client_secret_id]
        )
        return self._delete(request).transform(mapDashboardOrganizationsOauthAppsClientSecretsDeleteOutput.from_dict)


from typing import Any, Dict, List, Optional, Union
from metorial._endpoint import (
  BaseMetorialEndpoint,
  MetorialEndpointManager,
  MetorialRequest,
)
from ..resources import (
  mapDashboardInstanceSessionsConnectionsListOutput,
  DashboardInstanceSessionsConnectionsListOutput,
  mapDashboardInstanceSessionsConnectionsListQuery,
  DashboardInstanceSessionsConnectionsListQuery,
  mapDashboardInstanceSessionsConnectionsGetOutput,
  DashboardInstanceSessionsConnectionsGetOutput,
)


class MetorialManagementInstanceSessionsConnectionsEndpoint(BaseMetorialEndpoint):
  """Each time a new MCP connection to a server is established, a session connection is created. This allows you to track and manage the connections made during a session."""

  def __init__(self, config: MetorialEndpointManager):
    super().__init__(config)

  def list(
    self,
    instance_id: str,
    session_id: str,
    *,
    limit: Optional[float] = None,
    after: Optional[str] = None,
    before: Optional[str] = None,
    cursor: Optional[str] = None,
    order: Optional[str] = None
  ) -> DashboardInstanceSessionsConnectionsListOutput:
    """
    List session connections
    List all session connections

    :param instance_id: str
    :param session_id: str
    :param limit: Optional[float] (optional)
    :param after: Optional[str] (optional)
    :param before: Optional[str] (optional)
    :param cursor: Optional[str] (optional)
    :param order: Optional[str] (optional)
    :return: DashboardInstanceSessionsConnectionsListOutput
    """
    # Build query parameters from keyword arguments
    query_dict = {}
    if limit is not None:
      query_dict["limit"] = limit
    if after is not None:
      query_dict["after"] = after
    if before is not None:
      query_dict["before"] = before
    if cursor is not None:
      query_dict["cursor"] = cursor
    if order is not None:
      query_dict["order"] = order

    request = MetorialRequest(
      path=["instances", instance_id, "sessions", session_id, "connections"],
      query=query_dict,
    )
    return self._get(request).transform(
      mapDashboardInstanceSessionsConnectionsListOutput.from_dict
    )

  def get(
    self, instance_id: str, session_id: str, session_connection_id: str
  ) -> DashboardInstanceSessionsConnectionsGetOutput:
    """
    Get session connection
    Get the information of a specific session connection

    :param instance_id: str
    :param session_id: str
    :param session_connection_id: str
    :return: DashboardInstanceSessionsConnectionsGetOutput
    """
    request = MetorialRequest(
      path=[
        "instances",
        instance_id,
        "sessions",
        session_id,
        "connections",
        session_connection_id,
      ]
    )
    return self._get(request).transform(
      mapDashboardInstanceSessionsConnectionsGetOutput.from_dict
    )



from dataclasses import dataclass
from typing import Any, Dict, List, Optional, Union
from datetime import datetime
from metorial.utils import parse_iso_datetime
import dataclasses


@dataclass
class ManagementInstanceCustomServersListingGetOutputProfileBadges:
  type: str
  name: str


@dataclass
class ManagementInstanceCustomServersListingGetOutputProfile:
  object: str
  id: str
  name: str
  slug: str
  image_url: str
  is_official: bool
  is_metorial: bool
  is_verified: bool
  badges: List[ManagementInstanceCustomServersListingGetOutputProfileBadges]
  created_at: datetime
  updated_at: datetime
  description: Optional[str] = None


@dataclass
class ManagementInstanceCustomServersListingGetOutputCategories:
  object: str
  id: str
  name: str
  slug: str
  description: str
  created_at: datetime
  updated_at: datetime


@dataclass
class ManagementInstanceCustomServersListingGetOutputServer:
  object: str
  id: str
  name: str
  type: str
  created_at: datetime
  updated_at: datetime
  description: Optional[str] = None


@dataclass
class ManagementInstanceCustomServersListingGetOutputVendor:
  id: str
  identifier: str
  name: str
  image_url: str
  created_at: datetime
  updated_at: datetime
  description: Optional[str] = None
  attributes: Optional[Any] = None


@dataclass
class ManagementInstanceCustomServersListingGetOutputRepository:
  id: str
  identifier: str
  slug: str
  name: str
  provider_url: str
  website_url: str
  provider: str
  star_count: float
  fork_count: float
  watcher_count: float
  open_issues_count: float
  subscription_count: float
  default_branch: str
  license_name: str
  license_url: str
  license_spdx_id: str
  topics: List[str]
  created_at: datetime
  updated_at: datetime
  language: Optional[str] = None
  description: Optional[str] = None
  pushed_at: Optional[datetime] = None


@dataclass
class ManagementInstanceCustomServersListingGetOutputInstallation:
  id: str
  instance_id: str
  created_at: datetime


@dataclass
class ManagementInstanceCustomServersListingGetOutput:
  object: str
  id: str
  status: str
  slug: str
  image_url: str
  name: str
  description: str
  readme: str
  categories: List[ManagementInstanceCustomServersListingGetOutputCategories]
  skills: List[str]
  is_official: bool
  is_community: bool
  is_hostable: bool
  is_metorial: bool
  is_verified: bool
  server: ManagementInstanceCustomServersListingGetOutputServer
  created_at: datetime
  updated_at: datetime
  fork: Dict[str, Any]
  profile: Optional[ManagementInstanceCustomServersListingGetOutputProfile] = None
  vendor: Optional[ManagementInstanceCustomServersListingGetOutputVendor] = None
  repository: Optional[ManagementInstanceCustomServersListingGetOutputRepository] = None
  installation: Optional[
    ManagementInstanceCustomServersListingGetOutputInstallation
  ] = None
  oauth_explainer: Optional[str] = None
  readme_html: Optional[str] = None


class mapManagementInstanceCustomServersListingGetOutput:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> ManagementInstanceCustomServersListingGetOutput:
    return ManagementInstanceCustomServersListingGetOutput(
      object=data.get("object"),
      id=data.get("id"),
      status=data.get("status"),
      slug=data.get("slug"),
      image_url=data.get("image_url"),
      profile=mapManagementInstanceCustomServersListingGetOutputProfile.from_dict(
        data.get("profile")
      )
      if data.get("profile")
      else None,
      name=data.get("name"),
      description=data.get("description"),
      readme=data.get("readme"),
      categories=[
        mapManagementInstanceCustomServersListingGetOutputCategories.from_dict(item)
        for item in data.get("categories", [])
        if item
      ],
      skills=data.get("skills", []),
      is_official=data.get("is_official"),
      is_community=data.get("is_community"),
      is_hostable=data.get("is_hostable"),
      is_metorial=data.get("is_metorial"),
      is_verified=data.get("is_verified"),
      server=mapManagementInstanceCustomServersListingGetOutputServer.from_dict(
        data.get("server")
      )
      if data.get("server")
      else None,
      vendor=mapManagementInstanceCustomServersListingGetOutputVendor.from_dict(
        data.get("vendor")
      )
      if data.get("vendor")
      else None,
      repository=mapManagementInstanceCustomServersListingGetOutputRepository.from_dict(
        data.get("repository")
      )
      if data.get("repository")
      else None,
      installation=mapManagementInstanceCustomServersListingGetOutputInstallation.from_dict(
        data.get("installation")
      )
      if data.get("installation")
      else None,
      created_at=parse_iso_datetime(data.get("created_at"))
      if data.get("created_at")
      else None,
      updated_at=parse_iso_datetime(data.get("updated_at"))
      if data.get("updated_at")
      else None,
      fork=data.get("fork"),
      oauth_explainer=data.get("oauth_explainer"),
      readme_html=data.get("readme_html"),
    )

  @staticmethod
  def to_dict(
    value: Union[ManagementInstanceCustomServersListingGetOutput, Dict[str, Any], None]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    # assume dataclass for generated models
    return dataclasses.asdict(value)



from typing import Any, Dict, List, Optional, Union
from metorial._endpoint import (
  BaseMetorialEndpoint,
  MetorialEndpointManager,
  MetorialRequest,
)
from ..resources import (
  mapCustomServersManagedServerTemplatesListOutput,
  CustomServersManagedServerTemplatesListOutput,
  mapCustomServersManagedServerTemplatesListQuery,
  CustomServersManagedServerTemplatesListQuery,
  mapCustomServersManagedServerTemplatesGetOutput,
  CustomServersManagedServerTemplatesGetOutput,
)


class MetorialCustomServersManagedServerTemplatesEndpoint(BaseMetorialEndpoint):
  """Get managed server template information"""

  def __init__(self, config: MetorialEndpointManager):
    super().__init__(config)

  def list(
    self,
    organization_id: str,
    *,
    limit: Optional[float] = None,
    after: Optional[str] = None,
    before: Optional[str] = None,
    cursor: Optional[str] = None,
    order: Optional[str] = None
  ) -> CustomServersManagedServerTemplatesListOutput:
    """
    List oauth connection templates
    List all oauth connection templates

    :param organization_id: str
    :param limit: Optional[float] (optional)
    :param after: Optional[str] (optional)
    :param before: Optional[str] (optional)
    :param cursor: Optional[str] (optional)
    :param order: Optional[str] (optional)
    :return: CustomServersManagedServerTemplatesListOutput
    """
    # Build query parameters from keyword arguments
    query_dict = {}
    if limit is not None:
      query_dict["limit"] = limit
    if after is not None:
      query_dict["after"] = after
    if before is not None:
      query_dict["before"] = before
    if cursor is not None:
      query_dict["cursor"] = cursor
    if order is not None:
      query_dict["order"] = order

    request = MetorialRequest(
      path=["dashboard", "organizations", organization_id, "managed-server-templates"],
      query=query_dict,
    )
    return self._get(request).transform(
      mapCustomServersManagedServerTemplatesListOutput.from_dict
    )

  def get(
    self, organization_id: str, managed_server_id: str
  ) -> CustomServersManagedServerTemplatesGetOutput:
    """
    Get oauth connection template
    Get the information of a specific oauth connection template

    :param organization_id: str
    :param managed_server_id: str
    :return: CustomServersManagedServerTemplatesGetOutput
    """
    request = MetorialRequest(
      path=[
        "dashboard",
        "organizations",
        organization_id,
        "managed-server-templates",
        managed_server_id,
      ]
    )
    return self._get(request).transform(
      mapCustomServersManagedServerTemplatesGetOutput.from_dict
    )



from typing import Any, Dict, List, Optional, Union
from metorial_util_endpoint import BaseMetorialEndpoint, MetorialEndpointManager, MetorialRequest
from ..resources import mapDashboardOrganizationsInvitesListOutput, DashboardOrganizationsInvitesListOutput, mapDashboardOrganizationsInvitesListQuery, DashboardOrganizationsInvitesListQuery, mapDashboardOrganizationsInvitesGetOutput, DashboardOrganizationsInvitesGetOutput, mapDashboardOrganizationsInvitesCreateOutput, DashboardOrganizationsInvitesCreateOutput, mapDashboardOrganizationsInvitesCreateBody, DashboardOrganizationsInvitesCreateBody, mapDashboardOrganizationsInvitesEnsureLinkOutput, DashboardOrganizationsInvitesEnsureLinkOutput, mapDashboardOrganizationsInvitesDeleteOutput, DashboardOrganizationsInvitesDeleteOutput, mapDashboardOrganizationsInvitesUpdateOutput, DashboardOrganizationsInvitesUpdateOutput, mapDashboardOrganizationsInvitesUpdateBody, DashboardOrganizationsInvitesUpdateBody

class MetorialManagementOrganizationInvitesEndpoint(BaseMetorialEndpoint):
    """Read and write organization invite information"""

    def __init__(self, config: MetorialEndpointManager):
        super().__init__(config)

    def list(self, *, limit: Optional[float] = None, after: Optional[str] = None, before: Optional[str] = None, cursor: Optional[str] = None, order: Optional[str] = None) -> DashboardOrganizationsInvitesListOutput:
        """
    List organization invites
    List all organization invites

    :param limit: Optional[float] (optional)
    :param after: Optional[str] (optional)
    :param before: Optional[str] (optional)
    :param cursor: Optional[str] (optional)
    :param order: Optional[str] (optional)
    :return: DashboardOrganizationsInvitesListOutput
    """
        # Build query parameters from keyword arguments
        query_dict = {}
        if limit is not None:
            query_dict["limit"] = limit
        if after is not None:
            query_dict["after"] = after
        if before is not None:
            query_dict["before"] = before
        if cursor is not None:
            query_dict["cursor"] = cursor
        if order is not None:
            query_dict["order"] = order

        request = MetorialRequest(
            path=['organization', 'invites'],
            query=query_dict
        )
        return self._get(request).transform(mapDashboardOrganizationsInvitesListOutput.from_dict)

    def get(self, invite_id: str) -> DashboardOrganizationsInvitesGetOutput:
        """
    Get organization invite
    Get the information of a specific organization invite

    :param invite_id: str
    :return: DashboardOrganizationsInvitesGetOutput
    """
        request = MetorialRequest(
            path=['organization', 'invites', invite_id]
        )
        return self._get(request).transform(mapDashboardOrganizationsInvitesGetOutput.from_dict)

    def create(self) -> DashboardOrganizationsInvitesCreateOutput:
        """
    Create organization invite
    Create a new organization invite


    :return: DashboardOrganizationsInvitesCreateOutput
    """
        request = MetorialRequest(
            path=['organization', 'invites']
        )
        return self._post(request).transform(mapDashboardOrganizationsInvitesCreateOutput.from_dict)

    def ensure_link(self) -> DashboardOrganizationsInvitesEnsureLinkOutput:
        """
    Ensure organization invite link
    Ensure the invite link for the organization


    :return: DashboardOrganizationsInvitesEnsureLinkOutput
    """
        request = MetorialRequest(
            path=['organization', 'invites', 'ensure']
        )
        return self._post(request).transform(mapDashboardOrganizationsInvitesEnsureLinkOutput.from_dict)

    def delete(self, invite_id: str) -> DashboardOrganizationsInvitesDeleteOutput:
        """
    Delete organization invite
    Remove an organization invite

    :param invite_id: str
    :return: DashboardOrganizationsInvitesDeleteOutput
    """
        request = MetorialRequest(
            path=['organization', 'invites', invite_id]
        )
        return self._delete(request).transform(mapDashboardOrganizationsInvitesDeleteOutput.from_dict)

    def update(self, invite_id: str, *, role: str) -> DashboardOrganizationsInvitesUpdateOutput:
        """
    Update organization invite
    Update the role of an organization invite

    :param invite_id: str
    :param role: str
    :return: DashboardOrganizationsInvitesUpdateOutput
    """
        # Build body parameters from keyword arguments
        body_dict = {}
        body_dict["role"] = role

        request = MetorialRequest(
            path=['organization', 'invites', invite_id],
            body=body_dict
        )
        return self._post(request).transform(mapDashboardOrganizationsInvitesUpdateOutput.from_dict)


from .code import *
from .create import *
from .delete import *
from .deployments import *
from .events import *
from .get import *
from .list import *
from .listing import *
from .managed_server_templates import *
from .remote_servers import *
from .update import *
from .versions import *



from typing import Any, Dict, List, Optional, Union
from metorial._endpoint import (
  BaseMetorialEndpoint,
  MetorialEndpointManager,
  MetorialRequest,
)
from ..resources import (
  mapProviderOauthConnectionTemplateListOutput,
  ProviderOauthConnectionTemplateListOutput,
  mapProviderOauthConnectionTemplateListQuery,
  ProviderOauthConnectionTemplateListQuery,
  mapProviderOauthConnectionTemplateGetOutput,
  ProviderOauthConnectionTemplateGetOutput,
  mapProviderOauthConnectionTemplateEvaluateOutput,
  ProviderOauthConnectionTemplateEvaluateOutput,
  mapProviderOauthConnectionTemplateEvaluateBody,
  ProviderOauthConnectionTemplateEvaluateBody,
)


class MetorialProviderOauthConnectionTemplateEndpoint(BaseMetorialEndpoint):
  """Get OAuth connection template information"""

  def __init__(self, config: MetorialEndpointManager):
    super().__init__(config)

  def list(
    self,
    organization_id: str,
    *,
    limit: Optional[float] = None,
    after: Optional[str] = None,
    before: Optional[str] = None,
    cursor: Optional[str] = None,
    order: Optional[str] = None,
    profile_id: Optional[Union[List[str], str]] = None
  ) -> ProviderOauthConnectionTemplateListOutput:
    """
    List oauth connection templates
    List all oauth connection templates

    :param organization_id: str
    :param limit: Optional[float] (optional)
    :param after: Optional[str] (optional)
    :param before: Optional[str] (optional)
    :param cursor: Optional[str] (optional)
    :param order: Optional[str] (optional)
    :param profile_id: Optional[Union[List[str], str]] (optional)
    :return: ProviderOauthConnectionTemplateListOutput
    """
    # Build query parameters from keyword arguments
    query_dict = {}
    if limit is not None:
      query_dict["limit"] = limit
    if after is not None:
      query_dict["after"] = after
    if before is not None:
      query_dict["before"] = before
    if cursor is not None:
      query_dict["cursor"] = cursor
    if order is not None:
      query_dict["order"] = order
    if profile_id is not None:
      query_dict["profile_id"] = profile_id

    request = MetorialRequest(
      path=[
        "dashboard",
        "organizations",
        organization_id,
        "provider-oauth-connection-template",
      ],
      query=query_dict,
    )
    return self._get(request).transform(
      mapProviderOauthConnectionTemplateListOutput.from_dict
    )

  def get(
    self, organization_id: str, oauth_template_id: str
  ) -> ProviderOauthConnectionTemplateGetOutput:
    """
    Get oauth connection template
    Get the information of a specific oauth connection template

    :param organization_id: str
    :param oauth_template_id: str
    :return: ProviderOauthConnectionTemplateGetOutput
    """
    request = MetorialRequest(
      path=[
        "dashboard",
        "organizations",
        organization_id,
        "provider-oauth-connection-template",
        oauth_template_id,
      ]
    )
    return self._get(request).transform(
      mapProviderOauthConnectionTemplateGetOutput.from_dict
    )

  def evaluate(
    self, oauth_template_id: str, *, data: Dict[str, Any]
  ) -> ProviderOauthConnectionTemplateEvaluateOutput:
    """
    Evaluate oauth connection template
    Evaluate the configuration of an oauth connection template

    :param oauth_template_id: str
    :param data: Dict[str, Any]
    :return: ProviderOauthConnectionTemplateEvaluateOutput
    """
    # Build body parameters from keyword arguments
    body_dict = {}
    body_dict["data"] = data

    request = MetorialRequest(
      path=["provider-oauth-connection-template", oauth_template_id, "evaluate"],
      body=body_dict,
    )
    return self._post(request).transform(
      mapProviderOauthConnectionTemplateEvaluateOutput.from_dict
    )



from dataclasses import dataclass
from typing import Any, Dict, List, Optional, Union
from datetime import datetime
from metorial.utils import parse_iso_datetime
import dataclasses


@dataclass
class ManagementInstanceServerRunErrorsListOutputItemsServerRunServer:
  object: str
  id: str
  name: str
  type: str
  created_at: datetime
  updated_at: datetime
  description: Optional[str] = None


@dataclass
class ManagementInstanceServerRunErrorsListOutputItemsServerRunServerDeploymentServer:
  object: str
  id: str
  name: str
  type: str
  created_at: datetime
  updated_at: datetime
  description: Optional[str] = None


@dataclass
class ManagementInstanceServerRunErrorsListOutputItemsServerRunServerDeployment:
  object: str
  id: str
  metadata: Dict[str, Any]
  created_at: datetime
  updated_at: datetime
  server: ManagementInstanceServerRunErrorsListOutputItemsServerRunServerDeploymentServer
  name: Optional[str] = None
  description: Optional[str] = None


@dataclass
class ManagementInstanceServerRunErrorsListOutputItemsServerRunServerSessionMcpClient:
  object: str
  name: str
  version: str
  capabilities: Dict[str, Any]


@dataclass
class ManagementInstanceServerRunErrorsListOutputItemsServerRunServerSessionMcpServer:
  object: str
  name: str
  version: str
  capabilities: Dict[str, Any]


@dataclass
class ManagementInstanceServerRunErrorsListOutputItemsServerRunServerSessionMcp:
  object: str
  version: str
  connection_type: str
  client: Optional[
    ManagementInstanceServerRunErrorsListOutputItemsServerRunServerSessionMcpClient
  ] = None
  server: Optional[
    ManagementInstanceServerRunErrorsListOutputItemsServerRunServerSessionMcpServer
  ] = None


@dataclass
class ManagementInstanceServerRunErrorsListOutputItemsServerRunServerSessionUsage:
  total_productive_message_count: float
  total_productive_client_message_count: float
  total_productive_server_message_count: float


@dataclass
class ManagementInstanceServerRunErrorsListOutputItemsServerRunServerSession:
  object: str
  id: str
  status: str
  mcp: ManagementInstanceServerRunErrorsListOutputItemsServerRunServerSessionMcp
  usage: ManagementInstanceServerRunErrorsListOutputItemsServerRunServerSessionUsage
  session_id: str
  created_at: datetime


@dataclass
class ManagementInstanceServerRunErrorsListOutputItemsServerRun:
  object: str
  id: str
  type: str
  status: str
  server_version_id: str
  server: ManagementInstanceServerRunErrorsListOutputItemsServerRunServer
  server_deployment: ManagementInstanceServerRunErrorsListOutputItemsServerRunServerDeployment
  server_session: ManagementInstanceServerRunErrorsListOutputItemsServerRunServerSession
  created_at: datetime
  updated_at: datetime
  started_at: Optional[datetime] = None
  stopped_at: Optional[datetime] = None


@dataclass
class ManagementInstanceServerRunErrorsListOutputItems:
  object: str
  id: str
  code: str
  message: str
  metadata: Dict[str, Any]
  server_run: ManagementInstanceServerRunErrorsListOutputItemsServerRun
  created_at: datetime


@dataclass
class ManagementInstanceServerRunErrorsListOutputPagination:
  has_more_before: bool
  has_more_after: bool


@dataclass
class ManagementInstanceServerRunErrorsListOutput:
  items: List[ManagementInstanceServerRunErrorsListOutputItems]
  pagination: ManagementInstanceServerRunErrorsListOutputPagination


class mapManagementInstanceServerRunErrorsListOutputItemsServerRunServer:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> ManagementInstanceServerRunErrorsListOutputItemsServerRunServer:
    return ManagementInstanceServerRunErrorsListOutputItemsServerRunServer(
      object=data.get("object"),
      id=data.get("id"),
      name=data.get("name"),
      description=data.get("description"),
      type=data.get("type"),
      created_at=parse_iso_datetime(data.get("created_at"))
      if data.get("created_at")
      else None,
      updated_at=parse_iso_datetime(data.get("updated_at"))
      if data.get("updated_at")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[
      ManagementInstanceServerRunErrorsListOutputItemsServerRunServer,
      Dict[str, Any],
      None,
    ]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapManagementInstanceServerRunErrorsListOutputItemsServerRunServerDeploymentServer:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> ManagementInstanceServerRunErrorsListOutputItemsServerRunServerDeploymentServer:
    return (
      ManagementInstanceServerRunErrorsListOutputItemsServerRunServerDeploymentServer(
        object=data.get("object"),
        id=data.get("id"),
        name=data.get("name"),
        description=data.get("description"),
        type=data.get("type"),
        created_at=parse_iso_datetime(data.get("created_at"))
        if data.get("created_at")
        else None,
        updated_at=parse_iso_datetime(data.get("updated_at"))
        if data.get("updated_at")
        else None,
      )
    )

  @staticmethod
  def to_dict(
    value: Union[
      ManagementInstanceServerRunErrorsListOutputItemsServerRunServerDeploymentServer,
      Dict[str, Any],
      None,
    ]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapManagementInstanceServerRunErrorsListOutputItemsServerRunServerDeployment:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> ManagementInstanceServerRunErrorsListOutputItemsServerRunServerDeployment:
    return ManagementInstanceServerRunErrorsListOutputItemsServerRunServerDeployment(
      object=data.get("object"),
      id=data.get("id"),
      name=data.get("name"),
      description=data.get("description"),
      metadata=data.get("metadata"),
      created_at=parse_iso_datetime(data.get("created_at"))
      if data.get("created_at")
      else None,
      updated_at=parse_iso_datetime(data.get("updated_at"))
      if data.get("updated_at")
      else None,
      server=mapManagementInstanceServerRunErrorsListOutputItemsServerRunServerDeploymentServer.from_dict(
        data.get("server")
      )
      if data.get("server")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[
      ManagementInstanceServerRunErrorsListOutputItemsServerRunServerDeployment,
      Dict[str, Any],
      None,
    ]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapManagementInstanceServerRunErrorsListOutputItemsServerRunServerSessionMcpClient:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> ManagementInstanceServerRunErrorsListOutputItemsServerRunServerSessionMcpClient:
    return (
      ManagementInstanceServerRunErrorsListOutputItemsServerRunServerSessionMcpClient(
        object=data.get("object"),
        name=data.get("name"),
        version=data.get("version"),
        capabilities=data.get("capabilities"),
      )
    )

  @staticmethod
  def to_dict(
    value: Union[
      ManagementInstanceServerRunErrorsListOutputItemsServerRunServerSessionMcpClient,
      Dict[str, Any],
      None,
    ]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapManagementInstanceServerRunErrorsListOutputItemsServerRunServerSessionMcpServer:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> ManagementInstanceServerRunErrorsListOutputItemsServerRunServerSessionMcpServer:
    return (
      ManagementInstanceServerRunErrorsListOutputItemsServerRunServerSessionMcpServer(
        object=data.get("object"),
        name=data.get("name"),
        version=data.get("version"),
        capabilities=data.get("capabilities"),
      )
    )

  @staticmethod
  def to_dict(
    value: Union[
      ManagementInstanceServerRunErrorsListOutputItemsServerRunServerSessionMcpServer,
      Dict[str, Any],
      None,
    ]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapManagementInstanceServerRunErrorsListOutputItemsServerRunServerSessionMcp:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> ManagementInstanceServerRunErrorsListOutputItemsServerRunServerSessionMcp:
    return ManagementInstanceServerRunErrorsListOutputItemsServerRunServerSessionMcp(
      object=data.get("object"),
      version=data.get("version"),
      connection_type=data.get("connection_type"),
      client=mapManagementInstanceServerRunErrorsListOutputItemsServerRunServerSessionMcpClient.from_dict(
        data.get("client")
      )
      if data.get("client")
      else None,
      server=mapManagementInstanceServerRunErrorsListOutputItemsServerRunServerSessionMcpServer.from_dict(
        data.get("server")
      )
      if data.get("server")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[
      ManagementInstanceServerRunErrorsListOutputItemsServerRunServerSessionMcp,
      Dict[str, Any],
      None,
    ]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapManagementInstanceServerRunErrorsListOutputItemsServerRunServerSessionUsage:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> ManagementInstanceServerRunErrorsListOutputItemsServerRunServerSessionUsage:
    return ManagementInstanceServerRunErrorsListOutputItemsServerRunServerSessionUsage(
      total_productive_message_count=data.get("total_productive_message_count"),
      total_productive_client_message_count=data.get(
        "total_productive_client_message_count"
      ),
      total_productive_server_message_count=data.get(
        "total_productive_server_message_count"
      ),
    )

  @staticmethod
  def to_dict(
    value: Union[
      ManagementInstanceServerRunErrorsListOutputItemsServerRunServerSessionUsage,
      Dict[str, Any],
      None,
    ]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapManagementInstanceServerRunErrorsListOutputItemsServerRunServerSession:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> ManagementInstanceServerRunErrorsListOutputItemsServerRunServerSession:
    return ManagementInstanceServerRunErrorsListOutputItemsServerRunServerSession(
      object=data.get("object"),
      id=data.get("id"),
      status=data.get("status"),
      mcp=mapManagementInstanceServerRunErrorsListOutputItemsServerRunServerSessionMcp.from_dict(
        data.get("mcp")
      )
      if data.get("mcp")
      else None,
      usage=mapManagementInstanceServerRunErrorsListOutputItemsServerRunServerSessionUsage.from_dict(
        data.get("usage")
      )
      if data.get("usage")
      else None,
      session_id=data.get("session_id"),
      created_at=parse_iso_datetime(data.get("created_at"))
      if data.get("created_at")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[
      ManagementInstanceServerRunErrorsListOutputItemsServerRunServerSession,
      Dict[str, Any],
      None,
    ]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapManagementInstanceServerRunErrorsListOutputItemsServerRun:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> ManagementInstanceServerRunErrorsListOutputItemsServerRun:
    return ManagementInstanceServerRunErrorsListOutputItemsServerRun(
      object=data.get("object"),
      id=data.get("id"),
      type=data.get("type"),
      status=data.get("status"),
      server_version_id=data.get("server_version_id"),
      server=mapManagementInstanceServerRunErrorsListOutputItemsServerRunServer.from_dict(
        data.get("server")
      )
      if data.get("server")
      else None,
      server_deployment=mapManagementInstanceServerRunErrorsListOutputItemsServerRunServerDeployment.from_dict(
        data.get("server_deployment")
      )
      if data.get("server_deployment")
      else None,
      server_session=mapManagementInstanceServerRunErrorsListOutputItemsServerRunServerSession.from_dict(
        data.get("server_session")
      )
      if data.get("server_session")
      else None,
      created_at=parse_iso_datetime(data.get("created_at"))
      if data.get("created_at")
      else None,
      updated_at=parse_iso_datetime(data.get("updated_at"))
      if data.get("updated_at")
      else None,
      started_at=parse_iso_datetime(data.get("started_at"))
      if data.get("started_at")
      else None,
      stopped_at=parse_iso_datetime(data.get("stopped_at"))
      if data.get("stopped_at")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[
      ManagementInstanceServerRunErrorsListOutputItemsServerRun, Dict[str, Any], None
    ]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapManagementInstanceServerRunErrorsListOutputItems:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> ManagementInstanceServerRunErrorsListOutputItems:
    return ManagementInstanceServerRunErrorsListOutputItems(
      object=data.get("object"),
      id=data.get("id"),
      code=data.get("code"),
      message=data.get("message"),
      metadata=data.get("metadata"),
      server_run=mapManagementInstanceServerRunErrorsListOutputItemsServerRun.from_dict(
        data.get("server_run")
      )
      if data.get("server_run")
      else None,
      created_at=parse_iso_datetime(data.get("created_at"))
      if data.get("created_at")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[ManagementInstanceServerRunErrorsListOutputItems, Dict[str, Any], None]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapManagementInstanceServerRunErrorsListOutputPagination:
  @staticmethod
  def from_dict(
    data: Dict[str, Any]
  ) -> ManagementInstanceServerRunErrorsListOutputPagination:
    return ManagementInstanceServerRunErrorsListOutputPagination(
      has_more_before=data.get("has_more_before"),
      has_more_after=data.get("has_more_after"),
    )

  @staticmethod
  def to_dict(
    value: Union[
      ManagementInstanceServerRunErrorsListOutputPagination, Dict[str, Any], None
    ]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    return dataclasses.asdict(value)


class mapManagementInstanceServerRunErrorsListOutput:
  @staticmethod
  def from_dict(data: Dict[str, Any]) -> ManagementInstanceServerRunErrorsListOutput:
    return ManagementInstanceServerRunErrorsListOutput(
      items=[
        mapManagementInstanceServerRunErrorsListOutputItems.from_dict(item)
        for item in data.get("items", [])
        if item
      ],
      pagination=mapManagementInstanceServerRunErrorsListOutputPagination.from_dict(
        data.get("pagination")
      )
      if data.get("pagination")
      else None,
    )

  @staticmethod
  def to_dict(
    value: Union[ManagementInstanceServerRunErrorsListOutput, Dict[str, Any], None]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    # assume dataclass for generated models
    return dataclasses.asdict(value)


@dataclass
class ManagementInstanceServerRunErrorsListQuery:
  limit: Optional[float] = None
  after: Optional[str] = None
  before: Optional[str] = None
  cursor: Optional[str] = None
  order: Optional[str] = None
  server_session_id: Optional[Union[str, List[str]]] = None
  server_implementation_id: Optional[Union[str, List[str]]] = None
  server_deployment_id: Optional[Union[str, List[str]]] = None
  server_run_id: Optional[Union[str, List[str]]] = None
  server_run_error_group_id: Optional[Union[str, List[str]]] = None


class mapManagementInstanceServerRunErrorsListQuery:
  @staticmethod
  def from_dict(data: Dict[str, Any]) -> ManagementInstanceServerRunErrorsListQuery:
    return ManagementInstanceServerRunErrorsListQuery(
      limit=data.get("limit"),
      after=data.get("after"),
      before=data.get("before"),
      cursor=data.get("cursor"),
      order=data.get("order"),
      server_session_id=data.get("server_session_id"),
      server_implementation_id=data.get("server_implementation_id"),
      server_deployment_id=data.get("server_deployment_id"),
      server_run_id=data.get("server_run_id"),
      server_run_error_group_id=data.get("server_run_error_group_id"),
    )

  @staticmethod
  def to_dict(
    value: Union[ManagementInstanceServerRunErrorsListQuery, Dict[str, Any], None]
  ) -> Optional[Dict[str, Any]]:
    if value is None:
      return None
    if isinstance(value, dict):
      return value
    # assume dataclass for generated models
    return dataclasses.asdict(value)
